// devicelist.cc

#include "devicelist.h"
#include "eden.h"

#include <genesis/devices.h>
#include <cs/dataconn.h>
#include <cs/samlib.h>

#include <assert.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>


readclient::readclient()
{
    conn_ = NULL;
    lock_mask = 0;
}


writeclient::writeclient()
{
    conn_ = NULL;
    lock_mask = 0;
}

//----------------------- devicenode implementation ------------------

devicenode::devicenode(data_connection* conn, const char* owner, int dev_type, 
                       const char* desc, int nr_elements)
{
    int i;

    type_ = dev_type;
    strcpy(description_, desc);
    owner_ = strdup(owner);
    conn_ = conn;
    nr_elements_ = nr_elements;
    if (!(readclients_ = new (readclient*)[nr_elements_]))
        halt("devicenode::devicenode : couldn't allocate readclients_");
    if (!(writeclients_ = new (writeclient*)[nr_elements_]))
        halt("devicenode::devicenode : couldn't allocate writeclients_");

    for (i = 0; i < nr_elements_; i++)
    {
        readclients_[i] = NULL;
        writeclients_[i] = NULL;
    }

    readcount_ = 0;
    writecount_ = 0;

    demo=1;
    create_cookie();
}


devicenode::~devicenode()
{
    int i;

    for (i = 0; i < nr_elements_; i++)
    {
        if (readclients_[i] != NULL)
        {
            delete readclients_[i];
            readclients_[i] = NULL;
        }
        if (writeclients_[i] != NULL)
        {
            delete writeclients_[i];
            writeclients_[i] = NULL;
        }
    }

    delete[] readclients_;
    delete[] writeclients_;
    free(owner_);
    readcount_ = 0;
}


void devicenode::create_cookie()
{
    char* c = 0;
    if (demo) c = getenv("GENESIS_COOKIE");
    if (c) {
      if (strlen(c) != cookie_size) {
	cerr << "Error: illegal cookie: " << c << endl;
	cerr << "Error: need cookie of length: " << cookie_size << endl;
        exit(1);
      } else {
	sprintf(cookie_,c);
    	if (eden_trace) printf("new cookie = %s\n", cookie_);
        return;
      }
    } 
	

    for (int i = 0; i < cookie_size; i++)
    {
        int code = rand() % (26 + 26 + 10);

        if (code < 26)
            cookie_[i] = 'a' + code;
        else if (code < 52)
            cookie_[i] = 'A' + code - 26;
        else
            cookie_[i] = '0' + code - 52;
    }
    cookie_[cookie_size] = '\0';

    if (eden_trace) printf("new cookie = %s\n", cookie_);
}


int devicenode::writeslot_claimed()
{
    int i;

// find filled slot
    for (i = 0; i < nr_elements_; i++)
    {
        if (writeclients_[i] != NULL)
            return 1;
    }
    return 0;
}


int devicenode::readslot_claimed()
{
    int i;

// find filled slot
    for (i = 0; i < nr_elements_; i++)
    {
        if (readclients_[i] != NULL)
            return 1;
    }
    return 0;
}


int devicenode::writeslot_available()
{
    int i;
    int index = -1;

// find empty slot
    for (i = 0; i < nr_elements_; i++)
    {
        if (writeclients_[i] == NULL)
        {
            index = i;
            break;
        }
    }
    return index;
}


int devicenode::readslot_available()
{
    int i;
    int index = -1;

// find empty slot
    for (i = 0; i < nr_elements_; i++)
    {
        if (readclients_[i] == NULL)
        {
            index = i;
            break;
        }
    }
    return index;
}


int devicenode::add_readclient(data_connection* gclient)
{
    if ((type_ != dev_midi_in) && (type_ != dev_midi_inout))
        return 0;

    int index = readslot_available();

    if (index < 0)
        return 0;

    readclient* p = new readclient;
    p -> conn_ = gclient;
    readclients_[index] = p;
    readcount_++;

    if (eden_trace)
        cout << "added read client to device" << endl;

    return 1;
}


int devicenode::add_writeclient(data_connection* gclient)
{
    if ((type_ != dev_midi_out) && (type_ != dev_midi_inout))
        return 0;

    int index = writeslot_available();

    if (index >= 0)
    {
        writeclient* p = new writeclient;
        p -> conn_ = gclient;
        writeclients_[index] = p;
        writecount_++;
        return 0;
    }
    else
    {
        return -1;
    }
}


int devicenode::add_readwriteclient(data_connection* gclient)
{
    if (type_ != dev_midi_inout)
        return 0;

    int read_index = readslot_available();
    int write_index = writeslot_available();

    if ((read_index < 0) || (write_index < 0))
        return 0;

    readclient* pread = new readclient;
    pread -> conn_ = gclient;
    readclients_[read_index] = pread;
    readcount_++;

    writeclient* pwrite = new writeclient;
    pwrite -> conn_ = gclient;
    writeclients_[write_index] = pwrite;
    writecount_++;

    return 1;
}


void devicenode::remove_readclient(const data_connection* gclient)
{
    int i;

    for (i = 0; i < nr_elements_; i++)
    {
        if ((readclients_[i] != NULL) && (readclients_[i] -> conn_ == gclient))
        {
            delete readclients_[i];
            readclients_[i] = NULL;
            readcount_--;
        }
    }
    assert(readcount_ >= 0);
}


readclient* devicenode::nth_readclient(int n) const
{
    int i;
    int current = 0;

// since there can be 'holes' in the array, count from the beginning
// (is not very fast, but at least it works)
    for (i = 0; i < nr_elements_; i++)
    {
        if (readclients_[i] != NULL)
        {
            if (current == n)
                return readclients_[i];
            else
                current++;
        }
    }
// not found
    return NULL;
}


void devicenode::remove_writeclient(const data_connection* gclient)
{
    int i;

    for (i = 0; i < nr_elements_; i++)
    {
        if ((writeclients_[i] != NULL) && 
            (writeclients_[i] -> conn_ == gclient))
        {
            delete writeclients_[i];
            writeclients_[i] = NULL;
            writecount_--;
        }
    }
    assert(writecount_ >= 0);
}


writeclient* devicenode::nth_writeclient(int n) const
{
    int i;
    int current = 0;

// since there can be 'holes' in the array, count from the beginning
// (is not very fast, but at least it works)
    for (i = 0; i < nr_elements_; i++)
    {
        if (writeclients_[i] != NULL)
        {
            if (current == n)
                return writeclients_[i];
            else
                current++;
        }
    }
// not found
    return NULL;
}


void devicenode::remove_client(const data_connection* gclient)
{
    remove_readclient(gclient);
    remove_writeclient(gclient);
}


int devicenode::has_write_access(const data_connection* gclient) const
{
    int i;

    for (i = 0; i < nr_elements_; i++)
    {
        if ((writeclients_[i] != NULL) && 
            (writeclients_[i] -> conn_ == gclient))
        {
            return 1;
        }
    }

    return 0;
}


//-------------------- devicelist implementation ----------------

// The device id of a device is the index in list_

devicelist::devicelist(int nr_elements, int max_clients)
{
    int i;

    nr_elements_ = nr_elements;
    max_clients_ = max_clients;
    list_ = new (devicenode*)[nr_elements_];
    for (i = 0; i < nr_elements_; i++)
        list_[i] = NULL;

    srand(time(NULL));      // to generate random cookies
}


devicelist::~devicelist()
{
    int i;

    for (i = 0; i < nr_elements_; i++)
    {
        if (list_[i] != NULL)
        {
            delete list_[i];
            list_[i] = NULL;
        }
    }
    delete[] list_;
}


int devicelist::id_ok(int device_id) const
{
    return (list_[device_id] != NULL);
}


int devicelist::register_device(data_connection* device, int dev_type,
                                const char* desc, char* cookie, 
                                const char* username)
{
    int i;
    int index = -1;

// find an empty slot in list
    for (i = 0; i < nr_elements_; i++)
    {
        if (list_[i] == NULL)
        {
            index = i;
            break;
        }
    }
    if (index < 0)
        return -1;

    devicenode* p = new devicenode(device, username, dev_type, desc, 
                                   max_clients_);
    list_[index] = p;
    memcpy(cookie, list_[index] -> cookie(), cookie_size);

    return index;
}


void devicelist::unregister_device(int device_id)
{
    if (list_[device_id] != NULL)
    {
        delete list_[device_id];
        list_[device_id] = NULL;
    }
}


void devicelist::unregister_device(const data_connection* gdevice)
{
    int dev_id = device_id(gdevice);

    if (dev_id >= 0)
        unregister_device(dev_id);
}


data_connection* devicelist::device_conn(int device_id) const
{
    if (list_[device_id] != NULL)
        return list_[device_id] -> connection();
    else
        return NULL;
}


int devicelist::device_id(const data_connection* gdevice) const
{
    int i;

    for (i = 0; i < nr_elements_; i++)
    {
        if ((list_[i] != NULL) && (list_[i] -> connection() == gdevice))
            return i;
    }

// not found
    return -1;
}


int devicelist::request_device(data_connection* gclient, int dev_type,
                               const char* cookie, const char* username)
{
    int i;

    if (eden_trace)
        cout << "devicelist::request_device : cookie = " << cookie << endl;

// requesting a device with a cookie will always succeed if such a device
// exists, ie. you don't have to be the owner of that device
    if ((cookie != NULL) && (strlen(cookie) > 0))
    {
        int index = -1;
    // search device node with this cookie
        for (i = 0; i < nr_elements_; i++)
        {
            if ((list_[i] != NULL) && 
                (strncmp(cookie, list_[i] -> cookie(), cookie_size) == 0))
            {
                index = i;
                break;
            }
        }
        if (index < 0)
        {
            if (eden_trace)
                cout << "    cookie not found" << endl;
            return -1;
        }
    // try to insert client at possible devicenode
        if (dev_type == dev_midi_in)
        {
            if (list_[i] -> add_readclient(gclient) >= 0)
            {
                if (eden_trace)
                    cout << "added read client to device " << i << endl;
                return i;
            }
        }
        else if (dev_type == dev_midi_out)
        {
            if (list_[i] -> add_writeclient(gclient) >= 0)
            {
                if (eden_trace)
                    cout << "added write client to device " << i << endl;
                return i;
            }
        }
        else if (dev_type == dev_midi_inout)
        {
            if (list_[i] -> add_readwriteclient(gclient) >= 0)
            {
                if (eden_trace)
                    cout << "added read/write client to device " << i << endl;
                return i;
            }
        }
    }
    else
    {
        int best_found = -1;
        int least_nr_clients = 9999;

    // search appropriate devicenode with least number of clients attached
        for (i = 0; i < nr_elements_; i++)
        {
        // check for corresponding device types and ownership (for now,
        // only the owner of a device can obtain that device without using a
        // cookie)
            if ((list_[i] != NULL) && (list_[i] -> type() == dev_type) &&
                (strcmp(list_[i] -> owner(), username) == 0))
            {
                switch (dev_type)
                {
                    case dev_midi_in:
                        if (list_[i] -> nr_readclients() < least_nr_clients)
                        {
                            least_nr_clients = list_[i] -> nr_readclients();
                            best_found = i;
                        }
                        break;
                    case dev_midi_out:
                        if (list_[i] -> nr_writeclients() < least_nr_clients)
                        {
                            least_nr_clients = list_[i] -> nr_writeclients();
                            best_found = i;
                        }
                        break;
                    case dev_midi_inout:
                    // only count readclients with midi-in/out
                        if (list_[i] -> nr_readclients() < least_nr_clients)
                        {
                            least_nr_clients = list_[i] -> nr_readclients();
                            best_found = i;
                        }
                        break;
                } 
            }
        }

        if (best_found >= 0)
        {
        // try to insert client at possible devicenode
            switch (dev_type)
            {
                case dev_midi_in:
                    if (list_[best_found] -> add_readclient(gclient) >= 0)
                    {
                        if (eden_trace)
                        {
                            cout << "added read client to device ";
                            cout << best_found << endl;
                        }
                        return best_found;
                    }
                    else
                    {
                        if (eden_trace)
                        {
                            cout << "add_readclient for " << best_found;
                            cout << " failed" << endl;
                        }
                    }
                    break;
                case dev_midi_out:
                    if (list_[best_found] -> add_writeclient(gclient) >= 0)
                    {
                        if (eden_trace)
                        {
                            cout << "added write client to device ";
                            cout << best_found << endl;
                        }
                        return best_found;
                    }
                    else
                    {
                        if (eden_trace)
                        {
                            cout << "add_writeclient for " << best_found;
                            cout << " failed" << endl;
                        }
                    }
                    break;
                case dev_midi_inout:
                    if (list_[best_found] -> add_readwriteclient(gclient) >= 0)
                    {
                        if (eden_trace)
                        {
                            cout << "added readwrite client to device ";
                            cout << best_found << endl;
                        }
                        return best_found;
                    }
                    break;
            }
        }
        else
        {
            if (eden_trace)
                cout << "No device matched!" << endl;
        }
    }

    if (eden_trace)
        cout << "    just not successful" << endl;
    return -1;
}


void devicelist::release_device(const data_connection* gclient, int device_id)
{
    if (list_[device_id] != NULL)
        list_[device_id] -> remove_client(gclient);
}


int devicelist::has_device(int device_id) const
{
    return (list_[device_id] != NULL);
}


int devicelist::has_device(const data_connection* gdevice) const
{
    return (device_id(gdevice) >= 0);
}


int devicelist::nr_readclients(int device_id) const
{
    if (list_[device_id] != NULL)
        return list_[device_id] -> nr_readclients();
    else
        return -1;
}


readclient* devicelist::nth_readclient(int device_id, int n) const
{
    if (list_[device_id] != NULL)
        return list_[device_id] -> nth_readclient(n);
    else
        return NULL;
}


int devicelist::nr_writeclients(int device_id) const
{
    if (list_[device_id] != NULL)
        return list_[device_id] -> nr_writeclients();
    else
        return -1;
}


writeclient* devicelist::nth_writeclient(int device_id, int n) const
{
    if (list_[device_id] != NULL)
        return list_[device_id] -> nth_writeclient(n);
    else
        return NULL;
}


int devicelist::has_write_access(data_connection* gclient, int device_id) const
{
    if (list_[device_id] != NULL)
        return (list_[device_id] -> has_write_access(gclient));
    else
        return 0;
}


void devicelist::remove_client(data_connection* gclient)
{
    int i;

// search all devices that might possibly be granted to gclient
    for (i = 0; i < nr_elements_; i++)
    {
        if (list_[i] != NULL)
            list_[i] -> remove_client(gclient);
    }
}


//-------------------- devicelist_iterator implementation ----------------

devicelist_iterator::devicelist_iterator(devicelist* list, int device_id)
{
    current_ = 0;
    device_id_ = device_id;
    list_ = list;
}


void devicelist_iterator::first()
{
    current_ = 0;
}


void devicelist_iterator::next()
{
    current_++;
}


//-------------------- readlist_iterator implementation ----------------

readlist_iterator::readlist_iterator(devicelist* list, int device_id)
                 :devicelist_iterator(list, device_id)
{
}


int readlist_iterator::done()
{
    return current_ >= list_ -> nr_readclients(device_id_);
}


readclient* readlist_iterator::item() const
{
    return list_ -> nth_readclient(device_id_, current_);
}


//-------------------- writelist_iterator implementation ----------------

writelist_iterator::writelist_iterator(devicelist* list, int device_id)
                 :devicelist_iterator(list, device_id)
{
}


int writelist_iterator::done()
{
    return current_ >= list_ -> nr_writeclients(device_id_);
}


writeclient* writelist_iterator::item() const
{
    return list_ -> nth_writeclient(device_id_, current_);
}


