// jsmidi.cc

#include "misc.h"
#include "midi_in.h"
#include "midi_device.h"
#include "boolean.h"

#include <genesis/genesis.h>
#include <genesis/device.h>
#include <genesis/devices.h>
#include <cs/samlib.h>

#include <errno.h>
#include <fcntl.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/fcntl.h>

int trace_on = 0;

const char version_string[] = "JSMIDI/Genesis v0.4";

int main(int argc, char* argv[])
{
    int len;
    char buf[1024];
    char hostname[64] = EDEN_DEFAULT_HOST;
    int portnr = eden_default_port;
    //char inifile[128] = "/home/hush/lib/hush/midi/patch/ultrasnd.ini";
    char inifile[128] = "../patches/ultrasnd.ini";
 
    int argindex = 1;

    cout << endl;
    cout << version_string << "  -  by Hexagon & SAM" << endl << endl;

    while (argindex < argc)
    {
        if (strcmp(argv[argindex], "-host") == 0)
        {
            argindex++;
            if (argindex >= argc)
            {
                cerr << "Missing hostname" << endl;
                exit(1);
            }
            strcpy(hostname, argv[argindex]);
            argindex++;
        }
        else if (strcmp(argv[argindex], "-port") == 0)
        {
            argindex++;
            if (argindex >= argc)
            {
                cerr << "Missing portnumber" << endl;
                exit(1);
            }
            portnr = atoi(argv[argindex]);
            argindex++;
        }
        else if (strcmp(argv[argindex], "-trace") == 0)
        {
            argindex++;
            if (argindex >= argc)
            {
                cerr << "Missing trace level" << endl;
                exit(1);
            }
            trace_on = atoi(argv[argindex]);
            cout << "Tracing on, level " << trace_on << endl;
            argindex++;
        }
        else if (strcmp(argv[argindex], "-notrace") == 0)
        {
            cout << "Tracing off" << endl;
            trace_on = 0;
            argindex++;
        }
        else if (strcmp(argv[argindex], "-16") == 0)
        {
            cout << "Setting sample size to 16 bits" << endl;
            bps = 2;
            argindex++;
        }
        else if (strcmp(argv[argindex], "-8") == 0)
        {
            cout << "Setting sample size to 8 bits" << endl;
            bps = 1;
            argindex++;
        }
        else if (strcmp(argv[argindex], "-freq") == 0)
        {
            argindex++;
            if (argindex >= argc)
            {
                cerr << "Missing frequency" << endl;
                exit(1);
            }
            sample_freq = atoi(argv[argindex]);
            cout << "Setting sampling frequency to " << sample_freq;
            cout << " Hz" << endl;
            argindex++;
        }
        else if (strcmp(argv[argindex], "-dev") == 0)
        {
            argindex++;
            if (argindex >= argc)
            {
                cerr << "Missing device name" << endl;
                exit(1);
            }
            strcpy(audio_dev, argv[argindex]);
            cerr << "Setting audio device to " << audio_dev << endl;
            argindex++;
        }
        else if (strcmp(argv[argindex], "-inifile") == 0)
        {
            argindex++;
            if (argindex >= argc)
            {
                cerr << "Missing file name" << endl;
                exit(1);
            }
            strcpy(inifile, argv[argindex]);
            cout << "Initialization file set to " << inifile << endl;
            argindex++;
        }
        else
        {
            cerr << "Unknown option " << argv[argindex] << endl;
            cerr << "Possible options:" << endl;
            cerr << "    -host <hostname>" << endl;
            cerr << "    -port <portnr>" << endl;
            cerr << "    -trace" << endl;
            cerr << "    -notrace" << endl;
            cerr << "    -16" << endl;
            cerr << "    -8" << endl;
            cerr << "    -freq <samplefreq>" << endl;
            cerr << "    -dev <device>" << endl;
            cerr << "    -inifile <initialization file>" << endl;
            exit(1);
        }
    }

    boolean session_busy;

    init_pitch_table();
    midi_in* m = new midi_in(inifile);

    if (trace_on)
        cout << "Trying to connect to Eden..." << endl;
    genesis_device* device = new genesis_device(hostname, portnr,
                                                dev_midi_out, 
                                                version_string);
    if (!device -> register_dev())
    {
        cerr << "Couldn't register device" << endl;
        exit(1);
    }

    if (trace_on)
        cout << "device registered!" << endl;

    cout << "magic cookie: " << device -> cookie() << endl;

    session_busy = true;

    unbuffer_stdin();
    unbuffer_stdout();

    if (trace_on)
        cout << "entering mainloop..." << endl;
    while (session_busy)
    {
        fd_set readfds;

        FD_ZERO(&readfds);
        FD_SET(0, &readfds);
        FD_SET(device -> fd(), &readfds);

#ifdef i386
        timeval tv = {0, 0};

// Linux VoxWare will block until almost finished, so the timer is not used
        m -> device() -> play();
        if (select(device -> fd() + 1, &readfds, NULL, NULL, &tv) > 0)
#else
// Solaris should block
        if (select(device -> fd() + 1, &readfds, NULL, NULL, NULL) > 0)
#endif
        {
            if (FD_ISSET(0, &readfds))
            {
                char ch = getchar();
                switch(ch)
                {
                    case 's':
                        m -> silence();
                        break;
                    case 'r':
                        m -> reset();
                        break;
                    case 'q':
                        session_busy = false;
                        break;
                    default:
                        cerr << "press 's', 'r' or 'q'" << endl;
                }
            }
            else if (FD_ISSET(device -> fd(), &readfds))
            {
                len = device -> receive_midi(buf, sizeof(buf));

                if (len > 0)
                {
                    if (trace_on > 1)
                        cout << "processing %d bytes..." << endl;
                    else if (trace_on)
                        printf("*");

                    m -> receive_data(buf, len);
                }
                else
                {
                    if (trace_on > 1)
                    {
                        cout << "receive-midi returned " << len << endl;
                        cout << "Connection closed..." << endl;
                    }
                    session_busy = false;
                }
            }
        }
    }

    restore_stdin();

    return 0;
}
