// midi_device.cc

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/time.h>
#include <fcntl.h>
#include <signal.h>

#ifdef i386
#else
#include <multimedia/audio_encode.h>
#include <multimedia/libaudio.h>
#endif

#include "jsmidi.h"
#include "table.h"
#include "note.h"
#include "midi_device.h"


midi_device* m;

void getalarm(int /* i */)
{
    static int busy = 0;

    if (busy)
        return;
    busy = 1;
    m -> play();
    busy = 0;
}


midi_device::midi_device(int fd)
{
    int i, j;

    for (i = 0; i < 16; i++)
    {
        for (j = 0; j < 128; j++)
        {
            list_[i][j] = 0;
        }
    }
    for (i = 0; i < max_notes; i++)
    {
        notes_[i] = 0;
    }
    memset(buf16_, 0, sizeof(buf16_));
    memset(buf8_, 0, sizeof(buf8_));
 
    table_ = new table(264, 128);
    fd_ = fd;

    m = this;

// assumes all values in misc.h are set now
#ifndef i386
    buf_size = sample_freq / 50;
#endif
}
 
 
midi_device::~midi_device()
{
    int i;

    for (i = 0; i < max_notes; i++)
    {
        if (notes_[i])
        {
            delete notes_[i];
        }
    }
}


void midi_device::start() const
{
    if (trace_on)
        fprintf(stdout, "starting midi...\n");
    struct itimerval t1;
    struct sigaction a;

    t1.it_interval.tv_sec = 0;
    t1.it_interval.tv_usec = (1000 * buf_size / sample_freq) * 1000;
#ifdef i386
    t1.it_interval.tv_usec -= 100;
#endif
    t1.it_value.tv_sec = 0;
    t1.it_value.tv_usec = (1000 * buf_size / sample_freq) * 1000;

    if (trace_on)
    {
        fprintf(stderr, "buf_size = %d, usec = %ld\n", buf_size,
                (long) (t1.it_interval.tv_usec));
    }

    signal(SIGALRM, getalarm);
    sigaction(SIGALRM, 0, &a);
#ifdef i386
    a.sa_flags &= ~SA_ONESHOT;
    a.sa_flags |= SA_RESTART;
#else
    a.sa_flags &= ~SA_RESETHAND;
    a.sa_flags |= SA_RESTART;
#endif
    sigaction(SIGALRM, &a, 0);
    setitimer(ITIMER_REAL, &t1, 0);
}
 
 
void midi_device::load(int pgm, char* path)
{
    table_ -> add(pgm, path);
}
 

void midi_device::load_drum (int pitch, char* path)
{
    table_ -> add_drum(pitch, path);
}

 
void midi_device::prog_change(int ch, int pgm)
{
    pgm_[ch] = pgm;
}
 
 
void midi_device::note_on(int ch, int pitch, int vol)
{
    int i;

    if (vol == 0)
    {
        note_off(ch, pitch);
    /*
    // who cares about subtle release, let's kill the bastard!
        int i = list_[ch][pitch];
        if (i != 0)
        {
            delete notes_[i];
            notes_[i] = 0;
            list_[ch][pitch] = 0;
            if (trace_on > 0)
                printf("abrupt note off.\n");
        }
        */
        return;
    }
    if (list_[ch][pitch])
    {
        if (trace_on)
            fprintf(stderr, "killing previous note.\n");
        note_off(ch, pitch);
    }
    for (i = 0; i < max_notes; i++)
    {
        if (!(notes_[i]))
        {
            if (ch != 9)
            {
                notes_[i] = new note(table_, pgm_[ch], pitch, vol);
                list_[ch][pitch] = i+1;
                return;
            }
            else
            {
                notes_[i] = new drum_note(table_, pitch, vol);
                list_[ch][pitch] = i+1;
                return;
            }
        }
    }
    if (trace_on)
        fprintf(stdout, "couldn't allocate note.\n");
}
 
 
void midi_device::note_off(int ch, int pitch)
{
    int i;

    if ((i = (list_[ch][pitch])))
    {
        // delete notes_[i-1];
        // notes_[i-1] = 0;
        if (notes_[i-1])
            notes_[i-1] -> kill();
        list_[ch][pitch] = 0;
    }
}


// end_of_track kills all notes
void midi_device::end_of_track()
{
    int i, j;
    if (trace_on)
        fprintf(stdout, "end_of_track\n");

    for (i = 0; i < max_notes; i++)
    {
        if (notes_[i-1])
            notes_[i-1] -> kill();
    }

    for (i = 0; i < 16; i++)
    {
        for (j = 0; j < 128; j++)
        {
            list_[i][j] = 0;
        }
    }
}


void midi_device::play()
{
    int i;
 
    if (bps == 2)
        write(fd_, buf16_, buf_size * 2);
    else
        write(fd_, buf8_, buf_size);

    memset(buffer_, 0, buf_size*sizeof(long));
 
    for (i = 0; i < max_notes; i++)
    {
        if (notes_[i])
        {
            if (!(notes_[i] -> play(buffer_, buf_size)))
            {
                delete notes_[i];
                notes_[i] = 0;
            }
        }
    }

#ifdef i386
    if (bps == 2)
    {
        for (i = 0; i < buf_size; i++)
            buf16_[i] = buffer_[i] >> 11;
    }
    else
    {
        for (i = 0; i < buf_size; i++)
            buf8_[i] = (char)(buffer_[i] >> 19) + 128;
    }
#else
    if (bps == 2)
    {
        for (i = 0; i < buf_size; i++)
            buf16_[i] = buffer_[i] >> 11;
    }
    else
    {
        for (i = 0; i < buf_size; i++)
            buf8_[i] = audio_l2u(buffer_[i] << 5);
    }
#endif
}

