// midi_in.cc

#include "jsmidi.h"
#include "misc.h"

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
 

#ifdef i386
#include <sys/soundcard.h>
#define AUDIO "/dev/dsp"
#else
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#include <multimedia/audio_hdr.h>
#include <multimedia/audio_encode.h>
#define AUDIO "/dev/audio"
#endif

#include "misc.h"
#include "midi_device.h"
#include "midi_in.h"


midi_in::midi_in(const char* init_file)
{
//    fd_ = open(AUDIO, O_WRONLY | O_NONBLOCK);
    printf("Using device %s, %d bits, %ld Hz\n", audio_dev, 8*bps, 
           sample_freq);
#ifdef i386
// let Linux VoxWare block writes till almost done, thereby providing
// an accurate timing mechanism
    fd_ = open(audio_dev, O_WRONLY);
#else
// Solaris 
    fd_ = open(audio_dev, O_WRONLY | O_NONBLOCK);
#endif

    if (fd_ < 0)
    {
        fprintf(stderr, "couldn't open audio device %s\n", audio_dev);
        perror("midi_device");
        exit (-1);
    }

#ifdef i386
    int dsp_stereo = 0;
    int dsp_speed = sample_freq;
//    int dsp_format = (bps == 2) ? AFMT_S16_LE : AFMT_S8;
//    int dsp_format = 0;
//    int dsp_frag = 0x00020008;  /* 2 fragments of 2^8=256 bytes */
    int dsp_frag = 0x00020007;  /* 2 fragments of 2^7=128 bytes */
    buf_size = 128;
 
    if (ioctl(fd_, SNDCTL_DSP_SETFRAGMENT, &dsp_frag)==-1)
    {
        fprintf (stderr, "Unable to set fragment\n");
        perror ("midi_in");
        exit (-1);
    }
/*
    if (ioctl (fd_, SNDCTL_DSP_GETFMTS, &dsp_format)==-1)
    {
        fprintf (stderr, "Unable to get format mask\n");
        perror ("midi_in");
        exit (-1);
    }
    if (trace_on)
        fprintf(stdout, "format mask = %x\n", dsp_format);

    dsp_format = 0x00;
    if (ioctl(fd_, SNDCTL_DSP_SETFMT, &dsp_format) == -1)
    {
        fprintf (stderr, "Unable to get current format\n");
        perror ("midi_in");
        exit (-1);
    }
    if (trace_on)
        printf("current format = %x\n", dsp_format);

    dsp_format = (bps == 2) ? AFMT_S16_LE : AFMT_S8;
//    dsp_format = AFMT_S16_LE;
    if (ioctl (fd_, SNDCTL_DSP_SETFMT, &dsp_format)==-1)
    {
        fprintf (stderr, "Unable to set format\n");
        perror ("midi_in");
        exit (-1);
    }
 */
    int samplesize = (bps == 2) ? 16 : 8;
    if (ioctl(fd_, SNDCTL_DSP_SAMPLESIZE, &samplesize) == -1)
    {
        fprintf (stderr, "Unable to set samplesize\n");
        perror ("midi_in");
        exit (-1);
    }

    if (ioctl(fd_, SNDCTL_DSP_STEREO, &dsp_stereo)==-1)
    {
        fprintf (stderr, "Unable to set mono/stereo\n");
        perror ("midi_in");
        exit (-1);
    }
    if (ioctl (fd_, SNDCTL_DSP_SPEED, &dsp_speed)==-1)
    {
        fprintf (stderr, "Unable to set speed\n");
        perror ("midi_in");
        exit (-1);
    }

    long int fragmentsize;
    ioctl(fd_, SNDCTL_DSP_GETBLKSIZE, &fragmentsize);
    if (trace_on)
        printf("Fragment size is %ld\n", fragmentsize);

#else
    Audio_hdr header;
 
    if (audio_get_play_config(fd_, &header) < 0)
    {
        fprintf(stderr, "couldn't get header\n");
        perror ("midi_device");
        exit (-1);
    }
 
    if (trace_on)
    {
        fprintf(stdout, "sample rate %d, smp/u %d b/u %d ch %d enc %d\n",
           header.sample_rate, header.samples_per_unit, header.bytes_per_unit,
           header.channels, header.encoding);
    }
 
    header.sample_rate = sample_freq;
    header.samples_per_unit = 1;
    header.bytes_per_unit = bps;
    header.channels = 1;                    // Mono

// assumes that bps has been set correctly
    if (bps == 1)
        header.encoding = AUDIO_ENCODING_ULAW;
    else
        header.encoding = AUDIO_ENCODING_LINEAR;
        
    header.endian = AUDIO_ENDIAN_BIG;       // Why?
    header.data_size = AUDIO_UNKNOWN_SIZE;
 
    if (audio_set_play_config(fd_, &header) < 0)
    {
        fprintf(stderr, "couldn't set header\n");
        perror ("midi_device");
        exit (-1);
    }

// TEST:
    if (audio_get_play_config(fd_, &header) < 0)
    {
        fprintf(stderr, "couldn't get header\n");
        perror ("midi_device");
        exit (-1);
    }
 
    if (trace_on)
    {
        fprintf(stdout, "sample rate %d, smp/u %d b/u %d ch %d enc %d\n",
                header.sample_rate, header.samples_per_unit, 
                header.bytes_per_unit, header.channels, header.encoding);
    }
#endif

    device_ = new midi_device(fd_);

    fprintf(stderr, "Initializing, please wait");

    FILE* file;
    char buffer[1024];
    char name[128];
    char patchdir[128];
    int nr;
    int status = 0;
    
    if ((file = fopen(init_file, "rb")) == NULL)
    {
        fprintf(stderr, "\nCan't find initialization file '%s'\n", init_file);
        exit(1);
    }
    strcpy(patchdir, "./");

// NOTE: JSMIDI uses the old format of ultrasnd.ini files, with only
//       definitions of melodic patches en drum patches (no bank definitions)
    while (fgets(buffer, sizeof(buffer), file))
    {
        if (strncmp(buffer, "[Ultrasound]", 12) == 0)
        {
            fgets(buffer, sizeof(buffer), file);
            if (sscanf(buffer, "PatchDir=%s\n", patchdir) <= 0)
            {
                fprintf(stderr, "%s : strange line in section [Ultrasound] "
                        "(%s)\n", init_file, buffer);
            }
        // remove trailing '\n'
            if (patchdir[strlen(patchdir)-1] == '\n')
                patchdir[strlen(patchdir)-1] = '\0';
        // add '/'
            if (patchdir[strlen(patchdir)-1] != '/')
                strcat(patchdir, "/");
            if (trace_on)
                printf("Patch directory: %s\n", patchdir);
        }
        if (sscanf(buffer, "[%s Patches]", name) > 0)
        {
            if (strcmp(name, "Melodic") == 0)
            {
                status = 1;
                if (trace_on)
                    printf("reading melodic patches...\n");
            }
            if (strcmp(name, "Drum") == 0)
            {
                status = 2;
                if (trace_on)
                    printf("reading drum patches...\n");
            }
        }
        if (sscanf(buffer, "%d=%s", &nr, name) == 2)
        {
            char fullname[256];
            strcpy(fullname, patchdir);
            strcat(fullname, name);
            switch (status)
            {
                case 1 : 
                    device_ -> load(nr, strcat(fullname, ".pat")); 
                    fprintf(stderr, ".");
                    break;
                case 2 : 
                    device_ -> load_drum(nr, strcat(fullname, ".pat")); 
                    fprintf(stderr, ".");
                    break;
            }
        }
    }

    fprintf(stderr, "\nDone!\n");

#ifndef i386
// Linux VoxWare doesn't use the timer, Solaris does
    device_ -> start();
#endif
    device_ -> play();
    device_ -> play();
}


void midi_in::note_off(int channel, char* &data)
{
    int pitch = *(data++);
    int vol   = *(data++);

    if (trace_on > 1)
        fprintf(stdout, "note_off(%d, %d, %d)\n", channel, pitch, vol);
    device_ -> note_off(channel, pitch);
}


void midi_in::note_on(int channel, char* &data)
{
    int pitch = *(data++);
    int vol   = *(data++);

    if (trace_on > 1)
        fprintf(stdout, "note_on(%d, %d, %d)\n", channel, pitch, vol);
    device_ -> note_on(channel, pitch, vol);
}


void midi_in::pgm_change(int channel, char* &data)
{
    int pgm = *(data++);

    if (trace_on > 1)
        fprintf(stdout, "pgm(%d, %d)\n", channel, pgm);
    device_ -> prog_change(channel, pgm);
}


void midi_in::meta_command(int /* command */, char* &data)
{
    int type = *(data++);
    int length = *(data++);

    if (type == 0x2f)       // End of Track
    {
        if (trace_on)
            fprintf(stdout, "End of Track\n");
        device_ -> end_of_track();
    }
    else                    // skip data
    {
        if (trace_on)
            fprintf(stdout, "other meta-event %d\n", type);
        for (int i = 0; i < length; i++)
            data++;
    }
}


void midi_in::receive_data(const char* data, int len)
{
    char* p = (char*) data;
    unsigned char command, channel;
    unsigned char full_command;

    while (p < (data + len))
    {
        full_command = *p;
        command = (*p) & 0xf0;
        channel = (*p) & 0x0f;
        p++;

        switch (command)
        {
            case 0x80: note_off(channel, p); break;
            case 0x90: note_on(channel, p); break;
            case 0xc0: pgm_change(channel, p); break;
            case 0xf0: 
                if (trace_on)
                    fprintf(stdout, "maybe meta-event?\n");
                if (full_command == 0xff)
                {
                    if (trace_on)
                        fprintf(stdout, "YES, meta-event!\n");
                    meta_command(channel, p); 
                }
                else
                {
                    if (trace_on)
                    {
                        fprintf(stdout, "no, is event %d, command = %d\n", 
                                (int)full_command, (int)command);
                    }
                }
                break;
            default: 
                fprintf(stderr, "invalid MIDI command %x\n", command);
                break;
        }
    }
}


void midi_in::silence()
{
    device_ -> end_of_track();
}


void midi_in::reset()
{
    int i;

    silence();
    for (i = 0; i < nr_channels; i++)
        device_ -> prog_change(i, 0);
}

