// misc.c
// pc data conversion routines
// and other stuff

#include "misc.h"

#ifdef i386
int buf_size = 128;
long sample_freq = 8000;
int bps = 1;
char audio_dev[64] = "/dev/dsp";
#else
long sample_freq = 16000;
int buf_size = sample_freq / 50;
char audio_dev[64] = "/dev/audio";
int bps = 2;
#endif

long pitch_table[128];

long note_freq(int note_num)
{ // based on linux soundriver code
    int notes[] = {
        261632, 277189, 293671, 311132, 329632, 349232,
        369998, 391998, 415306, 440000, 466162, 493880 };

    int octave = note_num / 12;
    int note = note_num % 12;

    const int base_octave = 5;

    long freq = notes[note];

    if (octave < base_octave)
        freq >>= (base_octave - octave);
    else if (octave > base_octave)
        freq <<= (octave - base_octave);

  return freq;
}

void init_pitch_table()
{
    int i;

    for (i = 0; i < 128; i++)
    {
        pitch_table[i] = note_freq(i);
    }
}


unsigned short pctohs(unsigned short ns)
{
#ifdef i386
    return ns;
#else
    char hs[2];
    hs[0] = ((char *) &ns)[1];
    hs[1] = ((char *) &ns)[0];
    return *(short *) hs;
#endif
}


unsigned long pctohl(unsigned long nl)
{
#ifdef i386
    return nl;
#else
    char hl[4];
    hl[0] = ((char *) &nl)[3];
    hl[1] = ((char *) &nl)[2];
    hl[2] = ((char *) &nl)[1];
    hl[3] = ((char *) &nl)[0];
    return *(unsigned long *) hl;
#endif
}
