// note.cc

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <fcntl.h>

#include "jsmidi.h"
#include "misc.h"
#include "patch.h"
#include "table.h"
#include "note.h"


note::note()
{
    patch_ = 0;
    pitch_ = 0;
    killed_ = 0;

    o_ = 0;
    s_ = 0;
    max_ = 0;
    loop_ = 0;
    vol_ = 0;
    data8_ = 0;
    data16_ = 0;

    sustain_delay_ = (sample_freq / buf_size) * 2;
}


note::note(table* t, int pgm, int pitch, int vol)
{
    long freq = pitch_table[pitch];
 
    patch_ = t -> find(pgm, freq);
    if (!patch_)
    {
        fprintf(stderr, "no patch for note (%d, %d, %d)\n", pgm, pitch, vol);
        return;
    }
    pitch_ = pitch;
 
    s_ = ((freq * 256) / sample_freq) * (patch_ -> sample()) /
         (patch_ -> base());
 
    o_ = 0;
 
    max_  = (patch_ -> loop_end()) << 8;
    loop_ = ((patch_ -> loop_end()) - (patch_ -> loop_start())) << 8;
 
    data8_  = (char*) (patch_ -> data());
    data16_ = (short*) (patch_ -> data());
 
    vol_ = vol;
 
    if ((patch_ -> mode()) & WAVE_16_BITS)
    {
        max_  >>= 1;
        loop_ >>= 1;
    }
    killed_ = 0;

    sustain_delay_ = (sample_freq / buf_size) * 2;
}
 
 
note::~note()
{
}
 
 
int note::play(long* buffer, int len)
{
    int i;
 
    if (!patch_) return 0;
    if (killed_) vol_ -= killed_;
    if (vol_ <= 0) return 0;

    if (sustain_delay_ == 0)
        killed_ = 1;
    else
        sustain_delay_--;

    if ((patch_ -> mode()) & WAVE_16_BITS)
    {
        for (i = 0; i < len; i++)
        {
            buffer[i] += ((long) data16_[o_>>8])*vol_;
            o_ += s_;
            if (o_ >= max_)
            {
                if (!((patch_ -> mode()) & WAVE_LOOPING))
                {
                    if (trace_on > 1)
                    {
                        fprintf(stdout, "killing note FAST! (vol was %d)\n", 
                                vol_);
                    }
                    killed_ = 32;
                    vol_ = 0;
                }
                o_ -= loop_;
            }
        }
    }
    else
    {
        for (i = 0; i < len; i++)
        {
            buffer[i] += (data8_[o_>>8] << 8) * vol_;
            o_ += s_;
            if (o_ >= max_) o_ -= loop_;
        }
    }
    return 1;
}


drum_note::drum_note(table* t, int pitch, int vol)
{
    long freq;

    patch_ = t -> find_drum(pitch);

    if (!patch_)
    {
        fprintf(stderr, "no patch for drum_note (%d, %d)\n", pitch, vol);
        return;
    }
    freq = patch_ -> base();
    pitch_ = 0;
 
    s_ = ((freq * 256) / sample_freq) * (patch_ -> sample()) /
         (patch_ -> base());
 
    o_ = 0;
 
    max_  = (patch_ -> loop_end()) << 8;
    loop_ = ((patch_ -> loop_end()) - (patch_ -> loop_start())) << 8;
 
    data8_  = (char*) (patch_ -> data());
    data16_ = (short*) (patch_ -> data());
 
    vol_ = vol;
 
    if ((patch_ -> mode()) & WAVE_16_BITS)
    {
        max_  >>= 1;
        loop_ >>= 1;
    }
    killed_ = 0;
}
 
