// test.cc

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>


#include "misc.h"
#include "midi_device.h"

#ifdef i386
#include <sys/soundcard.h>
#define AUDIO "/dev/audio"
#else
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#include <multimedia/audio_hdr.h>
#include <multimedia/audio_encode.h>
#define AUDIO "/dev/audio"
#endif


int main(int argc, char** argv)
{
    int fd;
    int i;

    fd = open(AUDIO, O_WRONLY); //  | O_NONBLOCK);
    if (fd < 0)
    {
        fprintf(stderr, "couldn't open audio device\n");
        return -1;
    }
#ifdef i386
    int dsp_stereo = 0;
    int dsp_speed = sample_freq;
    int dsp_format = AFMT_S16_LE;
    int dsp_frag = 0x00020008;  /* 2 fragments of 2^8=256 bytes */

    if (ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &dsp_frag)==-1)
    {
        fprintf (stderr, "%s: Unable to set fragment\n", argv[0]);
        perror (argv[0]);
        exit (-1);
    }
    if (ioctl (fd, SNDCTL_DSP_GETFMTS, &dsp_format)==-1)
    {
        fprintf (stderr, "%s: Unable to get format\n", argv[0]);
        perror (argv[0]);
        exit (-1);
    }
    fprintf(stdout, "format mask = %x\n", dsp_format);
    dsp_format = AFMT_S16_LE;
    if (ioctl (fd, SNDCTL_DSP_SETFMT, &dsp_format)==-1)
    {
        fprintf (stderr, "%s: Unable to set format\n", argv[0]);
        perror (argv[0]);
        exit (-1);
    }


    if (ioctl (fd, SNDCTL_DSP_STEREO, &dsp_stereo)==-1)
    {
        fprintf (stderr, "%s: Unable to set mono/stereo\n", argv[0]);
        perror (argv[0]);
        exit (-1);
    }
    if (ioctl (fd, SNDCTL_DSP_SPEED, &dsp_speed)==-1)
    {
        fprintf (stderr, "%s: Unable to set speed\n", argv[0]);
        perror (argv[0]);
        exit (-1);
    }
#else
    Audio_hdr header;

    if (audio_get_play_config(fd, &header) < 0)
    {
        fprintf(stderr, "couldn't get header\n");
        return -1;
    }

    fprintf(stdout, "sample rate %d, smp/u %d b/u %d ch %d enc %d\n",
           header.sample_rate, header.samples_per_unit, header.bytes_per_unit,
           header.channels, header.encoding);

    header.sample_rate = sample_freq;
    header.samples_per_unit = 1;
    header.bytes_per_unit = 1;
    header.channels = 1;
    header.encoding = AUDIO_ENCODING_ULAW;
    header.endian = AUDIO_ENDIAN_BIG;
    header.data_size = AUDIO_UNKNOWN_SIZE;
    header.data_size = 40;

    if (audio_set_play_config(fd, &header) < 0)
    {
        fprintf(stderr, "couldn't set header\n");
        return -1;
    }

    if (audio_get_play_config(fd, &header) < 0)
    {
        fprintf(stderr, "couldn't get header\n");
        return -1;
    }

    fprintf(stdout, "sample rate %d, smp/u %d b/u %d ch %d enc %d\n",
           header.sample_rate, header.samples_per_unit, header.bytes_per_unit,
           header.channels, header.encoding);
#endif
#if 0
    char buffer[100*1024];
    char *p = buffer;
    int len = 100*1024;
    int result;

    for (i = 0; i < 100*1024; i++)
    {
        // buffer[i] = audio_s2u((short) (32000.0 * sin(M_PI*((double) i)/32.0)));
        buffer[i] = ((char) (126.0 * sin(M_PI*((double) i)/64.0)));
        // printf("%3d\n", (int) buffer[i]);
    }
    while ((len > 0) && ((result = write(2, p, 256)) > 0))
    {
        len += result;
        p += result;
    }
    return -1;
#endif

    midi_device* m = new midi_device(fd);

    for (i = 1; i < argc; i++)
    {
        m -> load(i, argv[i]);
        m -> prog_change(i, i);
    }
    // m -> note_on(1, 44, 32000);
#if 0
    while (1)
    {
        fd_set readfds, writefds, exceptfds;
        timeval timeout;

        FD_ZERO(&readfds);
        FD_ZERO(&writefds);
        FD_ZERO(&exceptfds);

        // FD_SET(fd, &writefds);
        FD_SET(0, &readfds);

        timeout.tv_sec = 0;
        timeout.tv_usec = 0;

        select(fd, &readfds, &writefds, &exceptfds, &timeout);
        m -> play();
        if (FD_ISSET(fd, &writefds))
        {
            fprintf(stdout, "playing...\n");
        }
        if (FD_ISSET(0, &readfds))
        {
            char buf[512];
            int len;
            int ch, pitch, vol;
            unsigned current;
            audio_get_play_eof(fd, &current);

            fprintf(stdout, "reading data...eof = %u, %u\n", current, m -> cur());
            len = read(0, buf, 512);
            if (len < 0) return -1;
            buf[len] = 0;
            fprintf(stdout, "read \"%s\"\n", buf);
            if (sscanf(buf, "%d%d%d", &ch, &pitch, &vol) == 3)
            {
                fprintf (stdout, "m -> note_on(%d, %d, %d)\n", ch, pitch, vol);
                m -> note_on(ch, pitch, vol);
            }
        }
    }
#else
    // m -> play();
    // m -> play();
    m -> start();
    while (1)
    {
        char buf[512];
        int len;
        int ch, pitch, vol;
        unsigned current;
        audio_get_play_eof(fd, &current);

        len = read(0, buf, 512);
        if (len > 0)
        {
            buf[len] = 0;
            fprintf(stdout, "read \"%s\"\n", buf);
            if (sscanf(buf, "%d%d%d", &ch, &pitch, &vol) == 3)
            {
                fprintf (stdout, "m -> note_on(%d, %d, %d)\n", ch, pitch, vol);
                m -> note_on(ch, pitch, vol);
            }
        }
    }
#endif
    return 0;
}
