// table.cc

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <fcntl.h>

#include "patch_info.h"
#include "patch_header.h"
#include "sample_header.h"

#include "jsmidi.h"
#include "misc.h"
#include "patch.h"
#include "table.h"


table::table(int si, int sd)
{
    int i;

    list_ = new (patch*) [si];
    for (i = 0; i < si; i++)
    {
        list_[i] = 0;
    }
    size_ = si;

    drum_list_ = new (patch*) [sd];
    for (i = 0; i < sd; i++)
    {
        drum_list_[i] = 0;
    }
    drums_size_ = sd;
}
 
 
table::~table()
{
    int i;

    for (i = 0; i < size_; i++)
    {
        if (list_[i])
        {
            delete list_[i];
        }
    }
    delete[] list_;

    for (i = 0; i < drums_size_; i++)
    {
        if (drum_list_[i])
        {
            delete drum_list_[i];
        }
    }
    delete[] drum_list_;
}
 
 
void table::add_to_list(patch** list, int size, int pgm, char* path)
{
    int fd, i, j;
    char buf[1024];
    patch_header header;
    sample_header sample;
 
    patch* p;
 
    long offset = 0L;
 
    if ((fd = open (path, O_RDONLY, 0)) == -1)
    {
        perror(path);
        exit (-1);
    }
 
    if (read (fd, buf, 0xef) != 0xef)
    {
        fprintf (stderr, "%s: Short file\n", path);
        exit (-1);
    }
 
    memcpy((void *) &header, buf, sizeof(header));
 
// only test on first 9 characters of GF1PATCH110; old patches still work,
// but have the magic GF1PATCH100
    if (strncmp (header.magic, "GF1PATCH110", 9))
    {
        fprintf (stderr, "%s: Not a patch file %s\n", path, header.magic);
        exit (-1);
    }
 
    if (strncmp (header.version, "ID#000002", 10))
    {
        fprintf (stderr, "%s: Incompatible patch file version\n", path);
        exit (-1);
    }
 
    memcpy(&header.nr_waveforms, &buf[85], 2);
    memcpy(&header.master_volume, &buf[87], 2);
 
    header.nr_waveforms = pctohs(header.nr_waveforms);
    header.master_volume = pctohs(header.master_volume);
    header.data_size = pctohs(header.data_size);
 
    // header.nr_waveforms = *((unsigned short *) &buf[85]);
    // header.master_volume = *((unsigned short *) &buf[87]);
 
    if (trace_on > 5)
    {
        printf("Patch file: %s contains %d samples\n", path, 
               header.nr_waveforms);
        printf("Master volume: %d\n", header.master_volume);
    }
 
    offset = 0xef;
 
    for (i = 0; i < header.nr_waveforms; i++)
    {
        if (lseek (fd, offset, 0) == -1)
        {
            perror (path);
            exit (-1);
        }
 
        if (read (fd, &buf, sizeof(sample)) != sizeof(sample))
        {
            fprintf (stderr, "%s: Short file\n", path);
            exit (-1);
        }
 
        memcpy((void *) &sample, buf, sizeof(sample));
 
        sample.len = pctohl(sample.len);
        sample.loop_start = pctohl(sample.loop_start);
        sample.loop_end = pctohl(sample.loop_end);
 
        // Since some fields of the patch record are not 32bit aligned, we must
        // handle them specially.
 
        memcpy(&sample.base_freq, &buf[20], 2);
        memcpy(&sample.low_note, &buf[22], 4);
        memcpy(&sample.high_note, &buf[26], 4);
        memcpy(&sample.base_note, &buf[30], 4);
        memcpy(&sample.detune, &buf[34], 2);
 
        sample.base_freq = pctohs(sample.base_freq);
        sample.low_note = pctohl(sample.low_note);
        sample.high_note = pctohl(sample.high_note);
        sample.base_note = pctohl(sample.base_note);
        sample.detune = pctohs(sample.detune);
 
        if (trace_on > 5)
        {
            fprintf(stdout, "base freq = %d\n", sample.base_freq);
            fprintf(stdout, "low note %ld high note %ld base note %ld\n",
                    sample.low_note, sample.high_note, sample.base_note);
        }
 
        // memcpy(&sample.panning, &buf[36], 1);
        // sample.low_note = *(long *) &buf[22];
        // sample.high_note = *(long *) &buf[26];
        // sample.base_note = *(long *) &buf[30];
        // sample.detune = *(short *) &buf[34];
 
        sample.panning = (unsigned char) buf[36];
 
        memcpy (sample.envelope_rate, &buf[37], 6);
        memcpy (sample.envelope_offset, &buf[43], 6);
 
        sample.tremolo_sweep = (unsigned char) buf[49];
        sample.tremolo_rate = (unsigned char) buf[50];
        sample.tremolo_depth = (unsigned char) buf[51];
 
        sample.vibrato_sweep = (unsigned char) buf[52];
        sample.vibrato_rate = (unsigned char) buf[53];
        sample.vibrato_depth = (unsigned char) buf[54];
 
        sample.modes = (unsigned char) buf[55];
 
        memcpy(&sample.scale_frequency, &buf[56], 2);
        memcpy(&sample.scale_factor, &buf[58], 2);
 
        sample.scale_frequency = pctohs(sample.scale_frequency);
        sample.scale_factor = pctohs(sample.scale_factor);
 
        // sample.scale_frequency = *(short *) &buf[56];
        // sample.scale_factor = *(unsigned short *) &buf[58];
 
        offset += 96;
 
        if (lseek(fd, offset, 0) == -1)
        {
            perror(path);
            exit(-1);
        }
 
        p = new patch(fd, pgm, &sample);
 
        for (j = 0; j < size; j++)
        {
            if (list[j] == 0)
            {
                list[j] = p;
                break;
            }
        }
 
        if (j >= size)
        {
            fprintf (stderr, "%s: no space left in table (size = %d, "
                             "requested = %d, instrument = %d)\n", path, 
                             size, j, pgm);
            exit(-1);
        }
 
        offset += sample.len;
    }
    close(fd);
}
 
 
void table::add (int pgm, char* path)
{
    add_to_list(list_, size_, pgm, path);
}


void table::add_drum (int pitch, char* path)
{
    add_to_list(drum_list_, drums_size_, pitch, path);
}


void table::remove(int pgm)
{
    for (int i = 0; i < size_; i++)
    {
        if ((list_[i]) && (list_[i] -> pgm() == pgm))
        {
            delete list_[i];
            list_[i] = 0;
        }
    }
}
 

void table::remove_drum (int pitch)
{
    for (int i = 0; i < drums_size_; i++)
    {
        if ((drum_list_[i]) && (drum_list_[i] -> pgm() == pitch))
        {
            delete drum_list_[i];
            drum_list_[i] = 0;
        }
    }
}


patch* table::find(int pgm, long freq) const
{
    int best_diff = 256;
    int best_fit = -1;

    for (int i = 0; i < size_; i++)
    {
        patch* p = list_[i];
        int new_diff = 256;

        if (p)
        {
            new_diff = pgm - p -> pgm();
        // don't take instrumentnr higher than wanted nr
            if ((new_diff >= 0) && (new_diff < best_diff))
            {
                if ((p -> low() <= freq) && (p -> high() >= freq))
                {
                    best_diff = new_diff;
                    best_fit = i;
                }
            }
        }
    }
    if (best_fit > -1)
    {
        if (list_[best_fit] -> pgm() != pgm)
        {
            fprintf(stdout, "mapping instr. %d to %d\n", pgm, 
                    list_[best_fit] -> pgm());
        }
        return list_[best_fit];
    }
    else
        return 0;
}


patch* table::find_drum (int pitch) const
{
    for (int i = 0; i < drums_size_; i++)
    {
        if (drum_list_[i] && (drum_list_[i] -> pgm() == pitch))
            return drum_list_[i];
    }
    return 0;
}
