global keyboard hymne g_octave g_key_num
set keyboard(show_notes) 1
set keyboard(instrument) 1   	
set keyboard(white_width)  30
set keyboard(black_width)  16
set keyboard(black_length) 75    
set keyboard(sustain1) 100     
set keyboard(sustain2) 10 
set keyboard(sustain3) 0
set keyboard(note_names) [list C  C# D  D# E  F  F# G  G# A A# B ]
set keyboard(black_keys) [list    1     3        6     8    10   ]


proc keyboard::mk {w {N 16} {width -1} {height 150} {args ""} } {
	global keyboard
	if {($width != -1)} then {	
		set wwidth [expr $width / $N ]
	} else {
		set wwidth 30
		set width [expr $N * $wwidth ]
	}
	frame $w
	set hoff 30
	set keyboard(canvas) [canvas $w.c -width $width -height $height -bg black]
	keyboard::add_keys $keyboard(canvas) $N  $wwidth [expr $height - $hoff] $hoff
	
	set panel [button $keyboard(canvas).panel -text Panel \
	  -command "keyboard::pop_panel .keyb_top $keyboard(canvas).panel"]
#	$keyboard(canvas) create window 1 1 \
#		-anchor nw -window $panel -tags {control quit}
	pack $keyboard(canvas) -fill both -expand true
}

# Add N notes to canvas C
proc keyboard::add_keys {C {N 12} {width 200} {height 100} {hoff 30} } {
    global keyboard
  
    set white_length $height

    set nr_notes [llength $keyboard(note_names)]
    for {set count 0; set x 0; set wcount 0} {$wcount < $N} {incr count} {
        set octave  [expr $count / $nr_notes]
        set key_num [expr $count % $nr_notes]
        set black_index [lsearch -exact $keyboard(black_keys) $key_num]
        set is_black [expr $black_index != -1]
        set key($count) [lindex $keyboard(note_names) $key_num]
        if ($is_black) then {
            set id [$C create rectangle \
		$x $hoff [expr $x+$keyboard(black_width)] [expr $hoff+ $keyboard(black_length)] \
                    -fill black -outline white -width 1 \
                    -tags {black key} ]
        } else {
            set id [$C create rectangle \
		$x $hoff [expr $x+$keyboard(white_width)] [expr $hoff+$white_length] \
                    -fill white -outline black -width 1 \
                    -tags {white key} ]

            $C create text \
		[expr $x-3+$keyboard(white_width)/2] [expr $hoff + $white_length] \
                    -text "$key($count)" -anchor sw -tags {key white name}
            incr x $keyboard(white_width)
	    incr wcount
        }
        $C bind $id <Any-ButtonPress> "keyboard::key-press %b $octave $key_num"
        $C bind $id <Any-ButtonRelease> "keyboard::key-release %b"
        $C bind $id <Enter> "$C itemconfigure $id -width 3"
        $C bind $id <Leave> "$C itemconfigure $id -width 1"
    }
    $C raise black
    $C move black -[expr $keyboard(black_width)/1.75] 1
}

proc keyboard::key-press {button octave key_num} {
    global keyboard
    global g_octave
    global g_key_num
    set octave [expr $octave + 7 ]
    set strlen [string length $key_num]
    if {("$strlen" == 1 )} then { set key_num [join "0 $key_num" ""] }
    set sus $keyboard(sustain$button)
    if {($sus == 0)} then { set sus -1
    } else { set sus [expr $sus/10.0 ] }
#    puts "play -num i$keyboard(instrument) 0.0 $sus 0 $octave.$key_num"
#    play -num "i$keyboard(instrument) 0.0 $sus 0.0 $octave.$key_num"
    play "$octave.$key_num"
    set g_octave $octave
    set g_key_num $key_num
}

proc keyboard::key-release {button} {
    global keyboard
    global g_octave
    global g_key_num
    release "$g_octave.$g_key_num"
#    puts "release $g_octave.$g_key_num"
#    if {($keyboard(sustain$button) == 0)} then {
#        play -num "i$keyboard(instrument) 0 0 0 0.00"
#        puts "play -num i$keyboard(instrument) 0 0 0 0.00"
#    }
}

proc keyboard::pop_panel {w pbutton} {
	hymne::set_ports
	global keyboard
	toplevel $w
	wm title $w "Keyboard control panel"
	$pbutton configure -command "
		destroy $w
		$pbutton configure -command {keyboard::pop_panel $w $pbutton}
	"
	set width 500; set height 160

	set pcanv [canvas $w.c -width $width -height $height]
	set vol [scale $pcanv.volume  -label Volume -command hymne::set_volume \
			-orient horizontal -length [expr $width/2] ]
	for {set i 1} { $i < 4 } { incr i } {
		set swidth [expr $width/3 -10 ]
  	 	set sus($i) [scale $pcanv.sus$i \
  	      		-label "Sustain button $i"  \
  	      		-orient horizontal \
  	      		-command "set keyboard(sustain$i)" -length $swidth]
 	        $pcanv create window [expr 0 + ($swidth+10) * ($i-1)] $height\
  	      		-anchor sw -tags {panel sus} -window $sus($i)
	}	

	set pluck [radiobutton $pcanv.pluck \
    		-text pluck -variable keyboard(section) -relief flat \
		-value pluck \
		-command { 
			hymne::init pluck;
			play -reset 
			set keyboard(instrument) 1
		}]
	set fof [radiobutton $pcanv.fof \
    		-text fof -variable keyboard(section) -relief flat \
		-value fof \
		-command { 
			hymne::init fof;
			play -reset 
			set keyboard(instrument) 1
		}]
	set delay [radiobutton $pcanv.delay \
    		-text delay -variable keyboard(section) -relief flat \
		-value delay \
		-command { 
			hymne::init delay;
			play -reset 
			set keyboard(instrument) 1
		}]
	set chorus [radiobutton $pcanv.chorus \
    		-text chorus -variable keyboard(section) \
		-command {
			hymne::init default
			play -reset
			set keyboard(instrument) 3
		} -value chorus -relief flat ]
	set default [radiobutton $pcanv.default \
    		-text default -variable keyboard(section) \
		-state active -command {
			hymne::init default
			play -reset
			set keyboard(instrument) 1
		} -value default -relief flat ]
	set speaker [checkbutton $pcanv.speaker -variable hymne(speaker) \
    		-relief flat -text Speaker \
		-command {audio -speaker $hymne(speaker)
			hymne::set_ports} ]
	set headphone [checkbutton $pcanv.headphone -variable hymne(headphone)\
    		-relief flat -text Headphones \
		-command {audio -headphone $hymne(headphone)
			hymne::set_ports} ]
	set line_out [checkbutton $pcanv.line_out -variable hymne(line_out)\
    		-relief flat -text Line \
		-command {audio -line_out $hymne(line_out)
			hymne::set_ports} ]
	set show [checkbutton $pcanv.show \
   		 -relief flat -text {Show notes} -var keyboard(show_notes) \
		-command {
        	if {($keyboard(show_notes) == 1)} then { 
			$keyboard(canvas) raise name
       		} else {
			$keyboard(canvas) lower name
		}
    	} ]
 	set dismiss [button $pcanv.dm -text "Exit Panel" \
 		-command "destroy $w; $pbutton configure -command {keyboard::pop_panel $w $pbutton}" \
		-relief flat ]
	set exit_all [button $pcanv.exit -text {Exit All} \
		-command exit -relief flat ]
	$pcanv create window [expr (0*$width)/4] 1 \
		-anchor nw -window $default  -tags {panel default}
	$pcanv create window [expr (0*$width)/4] 40 \
		-anchor nw -window $delay  -tags {panel delay}
	$pcanv create window [expr (0*$width)/4] 20 \
		-anchor nw -window $chorus  -tags {panel chorus}
	$pcanv create window [expr (0*$width)/4] 40 \
		-anchor nw -window $delay  -tags {panel delay}
	$pcanv create window [expr (0*$width)/4] 60 \
		-anchor nw -window $fof  -tags {panel fof}
	$pcanv create window [expr (0*$width)/4] 80 \
		-anchor nw -window $pluck  -tags {panel pluck}
	$pcanv create window [expr (1*$width)/4] 1 \
		-anchor nw -window $speaker -tags {panel speaker}
	$pcanv create window [expr (1*$width)/4] 20 \
		-anchor nw -window $headphone -tags {panel headphone}
	$pcanv create window [expr (1*$width)/4] 40 \
		-anchor nw -window $line_out    -tags {panel line_out}
	$pcanv create window [expr (1*$width)/4] 60 \
		-anchor nw -window $show    -tags {panel show}
	$pcanv create window [expr (2*$width)/4] 1 \
		-anchor nw -window $vol -tags {panel vol}
	$pcanv create window [expr (2*$width)/4] [ expr 0.6*$height] \
		-anchor sw -window $dismiss -tags {panel dismiss}
	$pcanv create window [expr (3*$width)/4] [expr 0.6*$height] \
		-anchor sw -window $exit_all  -tags {panel exit}

	pack $pcanv -fill both -expand true

	if {($keyboard(instrument) == "3") } then {$chorus select} else {$chorus deselect}
	if {($keyboard(show_notes))}         then {$show select}   else {$show deselect}
	$sus(1) set $keyboard(sustain1)
	$sus(2) set $keyboard(sustain2)
	$sus(3) set $keyboard(sustain3)
	$vol set [ expr int( 100.5 * [audio -volume]) ]
	hymne::set_ports
}
