// Clef.h
//
// practicum OOP 1994 door M. van Welie & B. Schonhage
// 
// Deze klasse representeert een sleutel. Elke sleutel heeft een BaseLine
// die aangeeft op welke lijn de referentietoon ligt. Ook heeft elke sleutel
// een BasePitch die de toonhoogte van de referentietoon aangeeft.
// Een sleutel kan zichzelf laten zien op een canvas.
//

#ifndef CLEF_H
#define CLEF_H

#include <widgets/canvas.h>
#include <amuse/Pitch.h>
#include <amuse/Type.h>
#include <fstream.h>

enum Clefs {fClef,gClef,cClef,DrumClef};
const NrClefs=4;

class Clef {

public:
	Clef (canvas *Can,Clefs C);
	~Clef (void);

	void SetClefType(Clefs NewClef);		// Set the clef to a new type
	Clefs GetClefType(void);		// Returns the current cleftype
	Pitch* GetBasePitch(void);		// Returns the pitch of the baseline 
	int GetBaseLine(void);			// Returns the nr. of the baseline (0=top)
	
	void Save(ofstream& File); 
	int Load(ifstream& File); 
	void Show (int x,int y);		// Show the clef on position x,y on the canvas
	void Hide (void);			// Hides the clef from the canvas
	int Width(void);			// Returns the width of the current clef

	inline int operator==(const Clef &B) const {
		return (ClefType==B.ClefType);
	}

	Clef& operator=(const Clef&);		// Copy constructor
private:
	Clefs ClefType;
	Pitch* BasePitch;
	int BaseLine;
	canvas* Canvas;
	static long Counter;
	char Name[NameLength];
};

#endif
