// List.h
// A sorted lijst template written by M. van Welie & B. Schonhage (oop8)
// This lijst was inspired by Jaap's lijst template
//
// List elements should contain the operators '==' and '<'
// for sorting.

#ifndef LIJST_H
#define LIJST_H

template <class T>
struct node {
    T    *elt;
    node *next;
    node *prev;
};

template <class T>
class lijst {
public:
    lijst ();
    ~lijst ();

    lijst<T> &operator= (const lijst<T> &l);
    // copies a lijst into one other (usage l2 = l1; gives l2 is a copy of l1)

    void clear (void);
    // empties the whole lijst
    
    void insert (T *e);
    // inserts a copy of e to lijst and keeps lijst sorted

    int find (const T *e);
    // returns 1 if found, or 0 if not found.

    void remove (const T *e);
    // removes all elements equal to e

    T* operator[] (const int i) const;
    // gives a pointer to the i+1'th element (i from 0..count()-1)
    // use as array index (example: e = l [5]; // e is elt. 6.

    int count () const;
    // gives # elements in lijst (lijst = 0..count ()-1)

private:
    node<T>     *head;
    node<T>     *tail;
    node<T>	*current;
    int size;
};

template <class T>
lijst<T>::lijst ()
{
    head  = 0;
    tail  = 0;
    current = 0;
    size  = 0;
}

template <class T>
lijst<T>::~lijst ()
{
	clear();
}

template <class T>
void lijst<T>::clear(void)
{
	current=head;
	for(int i=0;i<size;i++) {
		current=current->next;
		delete head;
		head=current;
	}
	size=0;
	head=0;
	tail=0;
	current=0;
}

template <class T>
lijst<T> &lijst<T>::operator= (const lijst<T> &l)
{
    if (this == &l) return *this;

    clear();

    for (int i = 0; i < l.count (); i++)
    {
    	insert(l[i]);
    }
    return *this;
}

template <class T>
void lijst<T>::insert (T *e)
{
    node<T>     *p = new node<T>;
    
    p->elt = e;
    p->next = 0;
    p->prev = 0;

    if(size==0) {
    	head=p;
	tail=p;
	current=p;
	size=1;
	return;
    }
    find(e);	// sets current to position where e should be inserted
    if((*e)<(*current->elt)) {	// insert before current
	if(current==head) {
		p->next=head;
		head->prev=p;
		head=p;
	} else {
		p->prev=current->prev;
		p->next=current;
		current->prev->next=p;	
		current->prev=p;
	}
    } else {		// insert after current
	if(current==tail) {
		p->prev=tail;
		tail->next=p;
		tail=p;
	} else {
		p->next=current->next;
		p->prev=current;
		current->next->prev=p;
		current->next=p;
	}
    }
    size++;
    current=p;
}

template <class T>
int lijst<T>::find (const T *e)
{
	if (size==0) return 0;
	if ((*e)==(*current->elt)) return 1;

	if((*e)<(*current->elt)) {	// search left
		while((current!=head)&&((*e)<(*current->elt))) {
			current=current->prev;
			if((*e)==(*current->elt)) return 1;
		}
	} else {		// search right
		while((current!=tail)&&((*current->elt)<(*e))) {
			current=current->next;
			if((*e)==(*current->elt)) return 1;
		}
	}
	return 0;
}

template <class T>
void lijst<T>::remove (const T *e)
{
	node<T>	*OldCurrent;

	if(find(e)) {
//		cout << "Remove found an element."<<endl;
		OldCurrent=current;
		if (size==1) {
			head=0;
			tail=0;
			current=0;
		}
		else if(current==head) {
			head=current->next;
			current=head;
			head->prev=0;
		}
		else if(current==tail) {
			tail=current->prev;
			current=tail;
			tail->next=0;
		} else {	//remove element somewhere in the middle
			current->prev->next=current->next;
			current->next->prev=current->prev;
			current=current->next;
		}
		
		delete OldCurrent;
		size--;		
	}
 	else	cout << "Remove did not find an element."<<endl;
}

template <class T>
T* lijst<T>::operator[] (const int i) const
{
	node<T>     *q = head;

	if(i>=size) return tail->elt;

	for (int j=0;j!=i;j++) {
    		q = q -> next;
	}
	return q -> elt;
}

template <class T>
int lijst<T>::count () const
{
    return size;
}

#endif
