// Measure.h 
//
// practicum OOP 1994 door M. van Welie & B. Schonhage
//
// Deze klasse representeert een maat. Elke maat heeft een sleutel,
// maatsoort en signatuur. Elke maat kent ook zijn vorige 
// maat zodat sleutels, maatsoorten en signaturen alleen getoond worden
// als ze afwijken van de maat ervoor.
//

#ifndef MEASURE_H
#define MEASURE_H

#include <widgets/canvas.h>
#include <amuse/Type.h>
#include <amuse/Clef.h>
#include <amuse/Signature.h>
#include <amuse/Key.h>
#include <amuse/Object.h>
#include <amuse/Note.h>
#include <amuse/NoteTuple.h>
#include <amuse/Lijst.h>
#include <fstream.h> 

class Staff;

class Measure {
public:
	Measure(); 			// for list of measures
	Measure (canvas *Can,Staff *S,Clefs Ctype);       
	~Measure (void);		// NOTE: Users must maintain prev.

	int GetNrObjects();		// returns nr of object
	Object* SearchObject(char *ObjectName); // returns object on X,Y
	void SetStaffPtr(Staff *S);	// sets the staffptr of the measure
	void SetPrev(Measure *P);	// use this to set prev 
	void SetAttr(Measure *P);	// set the key, clef, sig like measure P 

	Staff* GetStaff();		// Returns the staff ptr

	Signature *GetSig (void);	// Returns the signature of this measure
	void SetSig (void);		// Set signature using SigPref object 
	Key *GetKey (void);		// Returns the key of this measure
	void SetKey (void);		// Set key using KeyPref object
	Clef* GetClef (void);		// Returns the clef of this measure
	void SetClef (void);		// Set clef using the ClefPref object
	int GetMinSize(void);		// Minimal own size
	int FreeSize(void);		// Size available for adding objects

	void AddObject (int X, int Y, ObjectType Type, 
		 	LengthType Length, StemType Stem, PointType Point);	
					// Add an object on pos (X,Y)
	void DeleteObject (char *Name);	// Delete object with name Name

	void InitGrouping(StemType Stem);	// Init grouping object
	int AddGrouping(char *Name);	// Add note to grouping, return 1 if succes
	void AcceptGrouping(void);	// Accept the current grouping
	
	void Calibrate(int Size);	// Calibrate the size of this measure to Size
	void Show(void);		// Show this measure on the canvas
	void Save(ofstream& File);	// Save this measure to FILE
	int Load(ifstream& File);	// Save this measure to FILE
	void Play(int NoteLength);
	void Move(int dx);		// Move this measure by dx pixels
	int Size(void);			// Real used size
	int In(int X);			// True if X is inside this measure
	void Clear();
	void GetXBorders(int &Left, int &Right);	// Sets Left and Right to the sides of the measure
	int Pitch2YCoord(Pitch *P);	// returns the y-pos of this pitch
	
	// operators needed for list template
	inline int operator==(const Measure &b) const {
		return (StartX==b.StartX);
	}

	inline int operator<(const Measure &b) const {
		return (StartX<b.StartX);
	}

	int CHANGED;
	lijst<Pitch> AltList;
protected:
	int StartX,EndX ; // Coordinates of a bar
private:
	Measure *Prev;		// Pointers to previous measure
	Staff *StaffPtr;	// Points to the staff it belongs to
	Signature *MeasureSignature;
	Key *MeasureKey;
	Clef *MeasureClef;
	lijst<Object> ObjectList;

	int NrGroup;
	Note *GroupList[MaxNrNotes];
	StemType GroupStem;
	static long Counter;
	canvas* Canvas;
	char Name[NameLength];

};

#endif
