// NoteTuple.h
//
// practicum OOP 1994 door M. van Welie & B. Schonhage
// 
// Deze klasse representeert een noot tupel ( een groep noten die
// verbonden zijn ) die in een maat staat. 
// Noten worden niet in het tuple gekopieerd, en moeten dus blijven
// bestaan. Ze worden alleen anders afgebeeld, nl. verbonden.
// Deze klasse is een object en erft dus diens members 
//

#ifndef NOTETUPLE_H
#define NOTETUPLE_H

#include <widgets/canvas.h>
#include <amuse/Object.h>
#include <amuse/Note.h>
#include <amuse/Type.h>

class NoteTuple : public Object {  

public:
	NoteTuple (canvas *C, Measure *M);
	NoteTuple (Object &O);
//	NoteTuple (const NoteTuple &N);	// Copy constructor
	~NoteTuple (void);

	void SetMeasure (Measure *M);	// Sets measure of the note
	void AddNote(Note *N);		// Adds a note to the tuple.
	Note *operator[] (int Index) const;	// Returns the note on the i'th pos
	int GetNrNotes(void);		// Returns the nr of notes in the tuple

	void SetStem(StemType NewStem);	// Set stem up or down
        StemType GetStem(void);		// Returns the stem

	void Save(ofstream& File);	// Save the note tuple
	int Load(ifstream& File);	// Load the note tuple
	void Show(void);		// Show the note tuple on the canvas
	void Hide(void);		// Hide the note tuple
	int Width(void);		// Returns the width of the note tuple
	int GetLength(void);		// returns the max. length of the notes
	SortType Type(void);		// returns notetuple
	
	NoteTuple& operator=(const NoteTuple&);	// assignment operator
	int operator==(const char *Name) const;
	int operator<(const Object &O) const;
	int operator==(const NoteTuple &N) const;
private:
	int NrNotes;
	int NrStemUp;
	StemType Stem;
	Note* NoteList[MaxNrNotes];
};

#endif
