// Object.h
//
// practicum OOP 1994 door M. van Welie & B. Schonhage
//
// Deze klasse is de basis (baseclass) van de objecten die op
// het scherm verschijnen zoals noten en rusten. Een object wordt op
// een relatieve plaats in de maat gezet.


#ifndef OBJECT_H
#define OBJECT_H

#include <widgets/canvas.h>
#include <amuse/Pitch.h>
#include <amuse/Type.h>
#include <fstream.h>
#include <string.h>

class Measure;

enum SortType {notetype, resttype, notetupletype, none};

class Object {

public:
	Object(canvas *C, Measure *M);
	Object();				// for list template
	Object(Object &O);
	virtual ~Object(void);

	void SetPos(int Rel, int Grid); 	// Positie in Rel/Grid
	void GetPos(int &Rel,int &Grid);
	int GetXPos(void); 			// X is de harde canvas coord.
	char *GetName(void);			// returns the name

	virtual void SetMeasure(Measure *M); 	// Zet pointer naar zijn maat.	
	Measure *GetMeasure(void);		// Geeft de pointer naar zijn maat
	virtual void Play(int) ;
	virtual void Save(ofstream& File);	// Save object 
	virtual int Load(ifstream& File);	// Load object 
	virtual void Show(void);		// toon het object op de canvas
	virtual void Hide(void);		// verberg object van de canvas
	virtual int Width(void); 		// Hoe breed is dit object ?
	virtual int GetLength(void);		// returns the length of note/rest
	virtual SortType Type(void);	// geeft het type object

	// operators needed for list template
	virtual int operator==(const Object &b) const;
 	virtual int operator<(const Object &b) const;
	virtual int operator==(const char *ObjName) const;
protected:
	int RelPos;
	int GridSize;
	int IsSelected;
	static long Counter;
	char Name[NameLength];
	canvas *Canvas;
	Measure *ObjectMeasure;
	SortType objecttype;
};

#endif
