// Staff.h 
//
// practicum OOP 1994 door M. van Welie & B. Schonhage
//
// Deze klasse representeert een notenbalk. Een notenbalk bevat
// weer maten. 
//

#ifndef STAFF_H
#define STAFF_H

#include <widgets/canvas.h>
#include <fstream.h>
#include <amuse/ScoreInfo.h>
#include <amuse/Type.h>
#include <amuse/Lijst.h>
#include <amuse/Pitch.h>
#include <amuse/Clef.h>
#include <amuse/Key.h>
#include <amuse/Measure.h>

class Staff {

public:
	Staff ();			// for list of staves
	Staff (ScoreInfo *S,canvas *Can,int starty,int NrLijnen);
	~Staff (void);

	void Save(ofstream& File);
	void Play(int NoteLength);
	void Calibrate(int MeasureNr, int Size);
	void Show();			// Show the staff
	int Size();			// Returns the height of the staff
	void Move(int dy);		// Move staff dy up/down
	int In(int Y);			// True if Y is inside this staff

	Measure* GetMeasure(int Nr);	// Return a pointer to measure Nr
	int GetNrLines (void);		// Returns the nr of lines
	int GetNrMeasures(void);	// Returns nr of measures it has
	int GetMinMeasureSize(int MeasureNr);	// returns minsize of m
	void GetYBorders(int &TopLine,int &BottomLine);
	void GetXBorders(int &Left,int &Right, int MeasureNr);

	int GetCenterLinePos(void);	// returns y pos of center line

	Measure *SearchMeasure(int X, int &MeasureNr );	// search measure

	int Pitch2YCoord(Clef* C,Pitch* P);	// Converts a pitch to a Y coordinate
	void YCoord2Pitch(int Y,Clef* C,Key *K,Pitch* P);	// Converts an Y coordinate to a pitch

	// Insert a copy of M in this staff 
	void InsertMeasure(Measure* &New,int CurrentNr,SideType Side); 	
	void RemoveMeasure (int MeasureNr);	// Remove measure MeasureNr from staff

	int TooWide(Measure* &M); 	// The measure that doesn't fit if exist
	int RemoveFirst(Measure* &M); 	// Remove first Measure if exist

	// operators needed for list template
	inline int operator==(const Staff &b) const {
		return (StartY==b.StartY);
	}

	inline int operator<(const Staff &b) const {
		return (StartY<b.StartY);
	}
	
	int CHANGED;
	ScoreInfo *SI;
    protected:
	int StartY,EndY; 
    private:
	int NrLines; 			// Nr of lines per staff
	int YLineCoord[MaxNrLines];	// Array of staffline coordinates
	lijst<Measure> MeasureList;	// List of measures
	canvas *Canvas;
//	ScoreInfo *SI;
	char Name[NameLength];
	static long Counter;

};

#endif

