// ToolBox.h
// 
// practicum OOP 1994 door M. van Welie & B. Schonhage
//
// Deze klasse verzorgt het keuzemenu. Het programma is altijd in een
// bepaalde functie-mode (zie FunctionType). Verder houdt het bij hoeveel
// worksheets er open zijn, wat het geselecteerde object is en welke bitmap
// bij de huidige functie hoort.
//

#ifndef TOOLBOX_H
#define TOOLBOX_H

#include <amuse/ButtonFile.h>
#include <amuse/Type.h>
#include <amuse/Object.h>
#include <string.h>
#include <widgets/toplevel.h>
#include <widgets/frame.h>
#include <widgets/button.h>
#include <widgets/radiobutton.h>
#include <widgets/message.h>

const int BMapNameLength = 50; 
const int NrRadioTypes = 4;

enum FunctionType { movemode, addmode, deletemode, 
		    groupmode, insertsystem, removesystem,insertmeasure,
		    removemeasure, clearmeasure, prefclef, prefkey, 
		    prefsig, addmeasure, addsystem, removemeasures	
   		  };

enum RadioType {object=1, alteration=2, stem=3, point=4};

class ToolBox : public toplevel {

public:
	ToolBox(char *WidgetName);
	~ToolBox(void);

	void ScoreMore(void);			// A new score has been created
	void ScoreLess(void);			// A score has been deleted
	int GetNrOpen(void);			// Gives the nr. of open scores 
	void SelectedObject(Object *Obj);	// Set the active object for modifications
   
	int operator()();
	void SetActive(int i);			// Sets the i'th button on active

   	const char* GetBitmapName(void);	// Get bitmap name of selected function
	FunctionType GetFunction(void);		// Get the selected function
	void SetFunction(FunctionType Func);	// Set the function of the ToolBox
	ObjectType GetType(void);		// Get the type of the object
   	LengthType GetLength(void);		// Get the length of the object (1/length)
	PointType GetPoint(void);		// Get the point of the object
   	StemType GetStemType(void);		// Get the stemtype of the object

private:
	frame *OptionFrame;
	message *Message;
	button *Button[NrButtons];
	radiobutton *NoteRadio[NrNoteButtons];
	radiobutton *RestRadio[NrNoteButtons];
	radiobutton *StemRadio[NrStemButtons];
	radiobutton *PointRadio[NrStemButtons];
	radiobutton *AltRadio[NrAltButtons];
	Object *Selected;
	int ActiveButton;
	int NrScores;
	StemType Stem;
	LengthType Length;
	PointType Point;
	FunctionType Function;
	char BMapName[BMapNameLength];

	void DrawOptionFrame(void);
	void SetBitmapName(void);
};


#endif
