// midi.h
// constants and functions for the MIDI protocol

#ifndef _MIDI_H_
#define _MIDI_H_


const int midi_nr_channels = 16;

const int midi_note_off = 0x80;
const int midi_note_on = 0x90;
const int midi_key_pressure = 0xa0;
const int midi_control = 0xb0;
const int midi_program = 0xc0;
const int midi_channel_pressure = 0xd0;
const int midi_pitch_wheel = 0xe0;

const int midi_system_exclusive = 0xf0;
const int midi_song_position = 0xf2;
const int midi_song_select = 0xf3;
const int midi_tune_request = 0xf6;
const int midi_eox = 0xf7;

const int midi_timing_clock = 0xf8;
const int midi_start = 0xfa;
const int midi_continue = 0xfb;
const int midi_stop = 0xfc;
const int midi_active_sensing = 0xfe;
const int midi_system_reset = 0xff;

const int midi_middle_C = 48;

const int midi_percussion_channel = 9;  // zero based!!

const int nr_instruments = 128;
const int nr_percussions = 128;

extern const char* instrument_name[nr_instruments];
extern const char* percussion_name[nr_percussions];

// some percussion instruments
enum midi_percussion
{
    midi_acoustic_bass_drum = 35,
    midi_acoustic_snare = 38,
    midi_hand_clap = 39,
    midi_closed_hihat = 42,
    midi_low_tom = 45,
    midi_open_hihat = 46,
    midi_crash_cymbal = 49,
    midi_high_tom = 50,
    midi_ride_cymbal = 51,
    midi_tambourine = 54,
};

// inline function for calculating a midi-note, given the octave number and
// the note
inline int midinote(int octave, int subnote)
    { return (octave * 12 + subnote); }


#endif _MIDI_H_
