// msgparser.h

#ifndef _MSGPARSER_H_
#define _MSGPARSER_H_

#include <genesis/midi.h>


class msgparser
{
public:
    int type(const char* msg) const;

    int create_request_dev(char* msg, int dev_type, 
                           const char* cookie = 0) const;
    int create_release_dev(char* msg, int dev_id) const;
    int create_grant_dev(char* msg, int dev_id) const;
    int create_reject_dev(char* msg) const;
    int create_lock_channels(char* msg, int dev_id, int chanmask) const;
    int create_unlock_channels(char* msg, int dev_id) const;
    int create_register_dev(char* msg, int dev_type, const char* desc) const;
    int create_unregister_dev(char* msg, int dev_id) const;
    int create_accept_dev(char* msg, int dev_id, const char* cookie) const;
    int create_midi(char* msg, int dev_id, const char* midi_data,
                    int midilen) const;
        // used by clients, device id controls to which device it is sent
    int create_midi(char* msg, const char* midi_data, int midilen) const;
        // used by devices (don't know any device id)
    int create_midi_multicast(char* msg, int dev_id, int* dev_ids, int nrdevs,
                              const char* midi_data, int midilen) const;
    int create_error(char* msg, int dev_id, int errno) const;

    int create_join_jam(char* msg, const char* outcookie = 0, 
                        const char* incookie = 0) const;
    int create_accept_jam(char* msg, int jam_id) const;
    int create_reject_jam(char* msg) const;
    int create_leave_jam(char* msg, int jam_id) const;
    int create_ask_users(char* msg) const;
    int create_give_users(char* msg, 
                          char* users[midi_nr_channels]) const;
        // note: users[9] is always NULL (percussion channel)

    int device_type(const char* msg) const;
    int device_id(const char* msg) const;
    int get_midi(const char* msg, int msglen, char* midi_data) const;
    int channel_mask(const char* msg, int msglen) const;
    int get_cookie(const char* msg, char* cookie) const;
    int parse_request_dev(const char* msg, int& dev_type, char* cookie,
                          char* username) const;
    int parse_register_dev(const char* msg, int& dev_type, char* desc,
                           char* username) const;
    int parse_midi_multicast(const char* msg, int& srcdev, 
                             int* dev_ids, int& nrdevs,
                             char* midi_data, int& midilen) const;

    int parse_join_jam(const char* msg, char* outcookie, char* incookie,
                       char* username) const;
    int parse_accept_jam(const char* msg, int& jam_id) const;
    int parse_leave_jam(const char* msg, int& jam_id) const;
    int parse_give_users(const char* msg, 
                         char* users[midi_nr_channels]) const;
        // note: users[9] is always NULL (percussion channel)

protected:
    void set_type(char* msg, int msgtype) const;
    void set_device_id(char* msg, int dev_id) const;
    void set_device_type(char* msg, int dev_type) const;
};


#endif _MSGPARSER_H_
