#include <grammar/scanner.h>
#include "assert.h"

#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "fstream.h"

extern "C" char *getcwd(char*);
extern "C" char *chdir(char*);
extern "C" void exit(int);

static char dirs[12][256];
static int ndir=0;

static char root[BUFSIZ];
static int nroot=0;

static int paths( int& argc, char **& argv)
{
char c;
char wd[512];

strcpy(dirs[ndir++],".");
getcwd(wd); strcpy(root,wd);
while ( (argc > 1 ) && ( argv[1][0] == '-' ) ) {
	c = argv[1][1];
	switch(c) {
		case 'I': strcpy(dirs[ndir++],argv[1]+2); break;
		case 'R': strcpy(root,argv[1]+2); break;
         	default: // cerr << "No option: " << c << endl; 
			 break;
		}
	argc--; argv++;
        }
chdir(root); getcwd(wd);
cerr << "ROOT: " << root << endl;
return 0;
}


static ifstream* find(char* s, char* ext="")
{
ifstream *is;
int i=0;  
char fs[BUFSIZ];

sprintf(fs,"%s%s",s,ext);

is = new ifstream();
is->open(fs,ios::in);
int fp = !is->fail();

if (!fp)
    while ( i < ndir ) {
	fs[0]='\0';
	if (s[0]=='/' || s[0]=='.' || s[0]=='~' ) {
		sprintf(fs,"%s%s",s,ext); i=ndir;
		}
	else
		sprintf(fs,"%s/%s%s",dirs[i],s,ext);
	is->open(fs,ios::in);
	fp = !is->fail();
	if (fp) i=ndir; else i++;
	}
//if (fp) cerr << "FILE: " << fs << endl;
if (is->fail()) cerr << "WARNING: " << fs << " DOES NOT EXIST" << endl;
//assert(!is->fail());
return is;
}

class lex_file {
ifstream* is;
char name[BUFSIZ];
public:
lex_file(char *s, char* ext ) : is ( find( s, ext ) ) { strcpy(name,s); }
~lex_file() { is->close();
//cerr << "Closing:" << name << endl;
if (is) delete is; }
istream* stream() {  return is; }

void process(iter<term>* l) { (*l)(); }
};

static int separator(char c) {
	return c == '/' || c == '~';
	}

static char* strip(char* s) {
char *b = s;
char *p = b;
	 while (*p != '\0') p++;
	 p--; while ( p != b && !separator(*p) ) p--;
	 if ( p != b ) p++;
	 return p;
	 }

class lex_directory {
char name[BUFSIZ];
char ext[8];
char wd[BUFSIZ];
char xd[BUFSIZ];
public:
lex_directory(char* s, char* x = "") { 
	strcpy(name,s);
	strcpy(ext,x);
	getcwd(wd);
	chdir(s);
	getcwd(xd);
	cerr << "CWD: " << xd << endl;
	}
~lex_directory() { chdir(wd); }
istream* stream() { lex_file f( strip(name), ext ); return f.stream(); }
void process(iter<term>* l) { (*l)(); }
};


static int unflatten(char* in,  char argv[12][80]) {
char* b = in;
while (*b == ' ') b++;
char* p = b;
int i = 0;
while ( *p != '\0' ) {
i++;
while ( *p != '\0' && *p != ' ' ) p++;
if ( *p == '\0' ) {
	strcpy(argv[i],b);
	return i;
} else {
	*p = '\0';
	strcpy(argv[i],b);
	p++;
	b = p;
	}
}
return i;
}



class lex : public scanner {
protected:
iter<term>* _yacc;
int so; int cd; int comment;
void unso() { so = 0; cd = 0; comment = 0; }
//int argc;
//char argv[12][80];

public:

lex(istream* in = &cin, ostream* out = &cout ) : scanner(in,out) {
	unso(); _yacc = 0;
	}
//~lex() { in << endl; }

term* operator()();

virtual void process(istream* xin) {
		lex l(xin);
		l();
		}

void paths(int& argc, char**& argv) { ::paths(argc,argv); }

void files(char* s, char* ext) {
int argc=0;
char argv[12][80];
cerr << "FILES: " << s << endl;
argc = unflatten(s,argv);
for( int i = 1; i <= argc; i++ ) {
	//cerr << "Reading:" << argv[i] << "***" << endl;
	(*yyout) << endl;
	soinc(argv[i],ext);
	(*yyout) << endl;
	//cerr << "end reading:" << argv[i] << "***" << endl;
	}
}


void directories(char* s, char* ext) {
int argc=0;
char argv[12][80];
//cerr << "DIRS: " << s << endl;
argc = unflatten(s,argv);
for( int i = 1; i <= argc; i++ ) {
	//cerr << "reading " << argv[i];
	cdinc(argv[i],ext);
	}
}
	
void soinc(char* s, char* ext = "") {
		lex_file f(s,ext);
		istream* xin = f.stream();
		this->process(xin);
		}

void cdinc(char* s, char* ext = "") {
		(*yyout) << endl;
		lex_directory f(s,ext);
		soinc( strip(s), ext);
		//istream* xin = f.stream();
		//this->process(xin);
		}
};

