#ifndef PARSER_H
#define PARSER_H

#include <hush/term.h>
#include <hush/iter.h>
#include <grammar/token.h>

extern "C" {  extern void free(void*); }

typedef struct { char *t_name; int t_val; } yytoktype;

#ifndef YYSTYPE
#define YYSTYPE token
#endif
			
class parser : public iter<term> {
protected:
iter<term>* yyiter;

typedef int yytabelem;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif

#if YYMAXDEPTH > 0
int yy_yys[YYMAXDEPTH];
YYSTYPE yy_yyv[YYMAXDEPTH];
#endif

int *yys;
YYSTYPE *yyv;

#ifdef YYNMBCHARS
int yycvtok(int i);
#endif

YYSTYPE *yypv;                   /* top of value stack */
int *yyps;                       /* top of state stack */

int yystate;                     /* current state */
int yytmp;                       /* extra var (lasts between blocks) */

int yynerrs;                    /* number of errors */

int yyerrflag;                  /* error recovery flag */
int yychar;                     /* current input token number */

// added for enabling garbage collection when sending a result
void release()  { free(yys); free(yyv); } 
// May be defined with -e option of ooyacc -- see documentation 
virtual void yyerror(char* s); 
public:
YYSTYPE yylval, yyval;

parser(iter<term>* l) : iter<term>(this) {
yyiter = l;
yychar = -1;
#if YYMAXDEPTH > 0
yys =  yy_yys;
yyv =  yy_yyv;
#endif
}

virtual int yylex() {
	//cerr << "GETTING ";
	term* x = (*yyiter)();
	//AE yylval = (YYSTYPE) *(token*)x;
	yylval = *(token*)x;
	delete x;// AE?
	//cerr << yylval << endl;
	return yylval;
	}

virtual term* operator()() {
	cerr << "ERROR: parser::()" << endl;
	return 0;
	}
int yydebug;
};

inline void parser::yyerror(char* s) { cerr << s; }
#endif
