#ifndef _DVBASELIST_H
#define _DVBASELIST_H

#include <hush/export.h>
#include <hush/iter.h>
#include <iostream.h>

// system level usage only.

class HUSH_API dvlist_type {
public:
  dvlist_type() { created++; }
  ~dvlist_type() { deleted++; }
public:
  static int created;
  static int deleted;

  static void* table;
  static void declare(const char*, void*, int flag = 0);
  static dvlist_type* lookup(const char*);
  static void destroy();
};

class HUSH_API dvsublist_type {
public:
  dvsublist_type() { created++; }
  virtual ~dvsublist_type() { deleted++; }
public:
  static int created;
  static int deleted;
};
 
class HUSH_API dvlistelement_type {
public:
  dvlistelement_type() { created++; }
  ~dvlistelement_type() { deleted++; }
public:
  static int created;
  static int deleted;
};

#include <hush/listelem.h>

class HUSH_API dvbasenode {
 public:
  dvbasenode(const void* e=0, dvbasenode *n =0, dvbasenode *p=0): el(e), next(n), prev(p) { created++; }
  ~dvbasenode() { deleted++; }
public:
  const void* el;
  dvbasenode* next;
  dvbasenode* prev;

  static int created;
  static int deleted;
};

class dvlist_type;

//// For a list with modalities and properties:

class HUSH_API dvbaselist {
  friend class dvbasenode;
public:
  typedef ostream& (*format_void)(ostream&, void*);
  typedef int (*compare_void)(const void*, const void*);
  typedef int (*lcompare_void)(const dvlist_type&,const dvlist_type&);
  typedef int (*predicate_void)(const void*, const dvlist_type&);
  typedef void* (*element_void)(const void*, const dvlist_type&);
  typedef void (*delete_void)(void*);
  typedef void* (*apply_void)(void*,const dvlist_type&);
  typedef void* (*map_void)(void*,const dvlist_type&);
  
  dvbaselist(const dvlist_type*);
  dvbaselist(dvbasenode* x, const dvlist_type*);
  ~dvbaselist();
  dvbaselist* clone() const;
  dvbaselist& operator=(const dvbaselist&);
  
  int empty() const;
  int length() const;
  int index() const;
  
  void* find( const void* el, int* before=0);
  void* next( const void* el );
  int contains(const void* el ) const;
  int count(const void* el) const;
  void insert(const void* el, int before_current=0);			// insert at proper place
  void linsert(const dvlist_type& l);
  void* replace(const void* el);
  void* replace(const void* el, int pos, int len=1);
  void  replace(const dvlist_type& l, int pos, int len=1);
  void* remove(); // remove current
  void  remove(int pos, int len=1);
  void  lremove(const dvlist_type& l);
  void  rotate(int i=1);
  
  //// Access
  
  void* head() const;
  void* last() const;
  
  iter<void>* iterator() const;
  
  //// Cursor
  
  void* current() const;
  void* current(int i);
  
  void reset();
  void* first();
  void* end();
  void* next();
  int more() const; 
  int less() const; 
  void* previous();
  
  //// Print
  
  void print(ostream&);
  
  void _register(void*) const;  /// garbage
  void l_register(void*) const; /// dvlist<T>
  void i_register(void*) const; /// iter<T>
  void s_register(void*) const; /// dvsublist<T>
  void e_register(void*) const; /// dvlistlement<T>
  
  //// Processing
  
  int process( void* f );
  dvlist_type* apply( void* f );
  dvlist_type* select(const void* el);
  
#ifdef recursive_templates
  dvlist_type* map( void* f);
#endif
  
  int refs;
  
protected:
  void init(const dvlist_type*); 
  void _insert(const void* el, int before);	// insert after (before) current
  
  //// The state
  int _len;			/// length
  int _index;			/// index of current
  dvbasenode* _first;
  dvbasenode* _last;
  dvbasenode* _current;

  const dvlist_type* _me; 	/// pointer to handle (real list)

  void* _garb;  /// garbage
  void* _lgarb; /// list
  void* _igarb; /// iter
  void* _sgarb; /// list
  void* _egarb; /// iter
  
public:
  static int created;
  static int deleted;
};

#endif
