
#ifndef _DICTIONARY_H
#define _DICTIONARY_H

#include <hush/iter.h>
#include <hush/string.h>
#include <hush/garbage.h>

#include <hush/export.h> // can this be refined?

// see man:dictionary(4),  //examples/hush/dictionary.h

class dvstring;

class HUSH_API dictionary_type {
public:
dictionary_type() { created++; }
virtual ~dictionary_type() { deleted++; }
static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static dictionary_type* lookup(const char*);
static void _destroy();
};

extern void set_dictionaryentry(void* raw, char* k, void* v);
extern void* get_dictionaryentry(void* raw, char* p);

//static dictionaryentry<void> dummy;


template<class T>
class dictionaryentry {
public:
dictionaryentry( void* raw, char* k ) : _k(k), _raw(raw){ }
T* operator=( T* v ) { set_dictionaryentry(_raw,_k,(void*)v); return (T*) v; }
operator T*() { return (T*) get_dictionaryentry(_raw,_k); }

private:
char* _k;
void* _raw;
};


class HUSH_API rawdictionary {
friend void set_dictionaryentry(void* raw, char* k, void* v);
friend void* get_dictionaryentry(void* raw, char* k);
public:
rawdictionary();
~rawdictionary();
void entry(char* k, void* v);
void* find(char* k);
void* del(char* k);
void destroy();

char* stats();
	
iter<dvstring>* search();
void* operator()(char* k);

dictionaryentry<void>* operator[](char* k);

void _register(void*);
void s_register(void*);
void e_register(void*);
void i_register(void*);
	
protected:
void* tbl;
void* _garb; /// garbage
void* _egarb; /// entry
void* _sgarb; /// string
void* _igarb; /// iter
void* _lgarb; /// lists ?
};


//// The dictionary class:

template<class T>
class dictionary : public dictionary_type {
public:
dictionary() { _garb = 0; d = new rawdictionary(); }
~dictionary() { delete d; if (_garb) delete _garb; }

void entry(char* k, T* v) { if (k && v) d->entry(k,v); }
T* find(char* k) { return (T*) (k?d->find(k):0); }
T* del(char* k) { return (T*) (k?d->del(k):0); }

void destroy() { d->destroy(); }

char* stats() { return d->stats(); }
	
//T* operator()(char* k) { return (T*) d->operator()(k); }

operator iter<dvstring>&() { return *(iter<dvstring>*) d->search(); }
iter<dvstring>& traverse() { return *(iter<dvstring>*) d->search(); }

dictionaryentry<T>& operator[](dvstring* k) { 
	return (*this)[(char*)*k];
	}

dictionaryentry<T>& operator[](const char* k) {
	void* x = 0;
	if (k) x = (*d)[ (char*) k];
	return * (dictionaryentry<T>*) x;
	}

void _register(const T* x) const {
		if (!_garb) ((dictionary<T>*)this)->_garb =
						new dvgarbage<T>;
		_garb->_register(x);
		}
	
protected:
rawdictionary* d;
dvgarbage<T>* _garb;
public:


static void declare(const char* s, void* h, int flag = 0) {
        dictionary_type::declare(s,h,flag);
        }
 
static dictionary<T>* lookup(const char* s) {
        return (dictionary<T>*) dictionary_type::lookup(s);
        }
 

};

#endif
