/**<hr>
The event class is crucial for hush and the
support for discrete event simulation.
It's implementation is a bit complex.
see event(4), <hush/hush.h>, <hush/handler.h>, <hush/kit.h>
<hr>**/

#ifndef HUSH_EVENT_H
#define HUSH_EVENT_H

#include <hush/export.h>


// class declarations

class widget;
class kit;
class event;
class histogram;
//</ignore>

// the event class -- a truly overloaded entity

#include <hush/handler.h>


// ? Will I be fat or lean ?

class HUSH_API event  : public handler {  /// B/W
int _del;
public:


// Event classes

enum { user = 0, system = 1, tcl, file, tt, Sim };

public: // should be protected, and may be will be

	event(kit*,int,char**);             // script (tcl)
	
	event(int qp=0,int sp=0,int k=0);   // event::_create  (sim)
	
	event(const char* s);               // modeling 
	event(const event&);
	
	// cloning
	
	virtual event* create(kit*,int,char**);
	
	// to allow for object references
	
	event(event*);                       
	virtual ~event();     
	
	// dispatching 

	virtual int operator()();

public:    // common methods 

virtual int eventclass() const ; /// defaults to 0 = user

virtual int type() const;   /// defaults to 0 

virtual char* name() const ; 

virtual void trace() const; 

// To obtain 'the' event, allows for 'type secure' event handling

virtual event* get() const;

// obsolete

virtual int no() const;
virtual char* id() const;

// etelosbo


virtual int argc() const;
virtual char** argv(int i = 0) const;
virtual char* arg(int i = 0) const;

virtual int fd() const;
virtual int mask() const;

// xevent methods -- for windows 

virtual int x() const ;
virtual int y() const ;
virtual int x_root() const ;
virtual int y_root() const ;
virtual int button(int i = 0) const ;
virtual int buttonup(int i = 0) const ;
virtual int keycode() const ;
virtual int buttonevent(int i = 0) const ;
virtual int keyevent() const ;
virtual int motion() const ;
virtual void motion( event& );
virtual void* rawevent() ;

// simevent -- for simulation 

virtual int report(histogram* hi,double interv = 0);
virtual void stamp();
virtual double timespent();

virtual char* reporttitle();
virtual double time() const;
virtual void time(double t);
virtual int schedulingpriority();
virtual void schedulingpriority(int p);
virtual int queuingpriority();
virtual void queuingpriority(int p);
virtual double interval();
virtual void interval(double i);
virtual int state();
virtual void state(int s);
virtual int killflag();
virtual event* leftpointer();
virtual void leftpointer(event*);
virtual event* middlepointer();
virtual void middlepointer(event*);
virtual event* rightpointer();
virtual void rightpointer(event*);
virtual event* parentpointer();
virtual void parentpointer(event*);
virtual double waitingreporttime();
virtual void waitingreporttime(double);
virtual double eventreporttime();
virtual int active();
virtual int pending();
virtual int conditional();
virtual int closed();
virtual int passive();
virtual int queued();


/// for sgml_events

virtual char* tag() const;
virtual char* attributes() const;
virtual char* data() const;



// the body --- also very weakly protected, still

private:
int exists() const; /// to check for circularity;
event* _body;
public:
static void* _last;    // to store the last event
static void* _create;  // to create sim_event body

static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static event* lookup(const char*);
static void destroy();
};

#endif
