///An item is a bit like a widget, but it lives unlike a
///widget on a canvas, whereas a widget lives on the screen.
///However, a widget can be embedded in an item.
/// man:item(4), <hush/hamdler.h> 

//<ignore>
#ifndef ITEMS_H
#define ITEMS_H

#include <hush/handler.h>
class kit;
class dvbinding;

#include <hush/export.h>
//</ignore>

class HUSH_API item : public handler {
public:
	item();
	item(const char*);
	item(item*);
	item(item&); /// copy constructor (should be const item&!)
	
	virtual ~item();
	
	virtual item& operator=(item&);  /// assignment (should be const item&!)

	
	virtual char* id() const ;
	virtual int no() const ; 

	char* name() const;
	
	operator int() const; /// calls no()
	
	item* operator ->(); /// non-virtual
	
	virtual void tag(const char* s);

	virtual char* tags() const;
	virtual char* coords() const;
	virtual char* bbox() const;
	
	virtual void configure(const char* s);
	virtual void move(int x, int y);
	virtual void del();
	
	virtual int bind(const char* s, handler* h, const char* args = "" );
	virtual int bind(const char* s, dvbinding* a, const char* args = "" );
	virtual int bind(handler* h, const char* args = "" );

	virtual int direct(const char* s, dvbinding* ac, const char* args);
	virtual class widget* widget();


protected:
	virtual void install(dvbinding* ac, const char* args = "");
private:
        int refs; 
	item* it;

public:
static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static item* lookup(const char*);
static void destroy();
};

//<ignore>
#endif
//</ignore>
