///The kit class defines the interface between C++ and Tcl.
///<i>hush</i>.
/// See also man:kit(4), <hush/handler.h>
/// //examples/hush/generate.c, <hush/kit.c> 

#ifndef KIT_H
#define KIT_H

#include <hush/export.h>

#include <hush/handler.h>

class dvbinding;
class widget;
class ostream;
class istream;


// Class interface

class HUSH_API kit : public handler {
protected:

public:

enum { tcl = 0, dlp, python, unknown };

kit(const char* = "tcl");
kit(kit*);
virtual ~kit();

// Evaluation:

virtual int eval(const char* cmd);
virtual char* evaluate(const char* cmd);

virtual char* result() const;
virtual void result(const char* s);

virtual int source(const char* f);

virtual int error(); /// N E W
virtual void error(const char*); // N E W

//virtual void after(int n, const char* cmd="");
//virtual void after(int n, void*, void* = 0);

virtual void after(int n, handler* h = 0);
virtual void* token() const;  /// Last TimerToken

//virtual int schedule(int period, handler* = 0, int cycles = 0, int slices = 0); // 0 = default!

virtual char* send(const char* it, const char* cmd);
virtual char* selection(const char* options="") const;
virtual void update(const char* options="");


virtual widget* root() const; 
virtual widget* pack(widget* w, const char* options = "-side top -fill x");
virtual widget* pack(const char* w, const char* options = "-side top -fill x");


// Binding Actions: high level:

enum {readable=1, writable=2, exception=4}; // jrvosse

virtual dvbinding* bind(const char* name, handler* h);
virtual dvbinding* bind(int fd, handler* h, int mask=readable); // jrvosse

virtual void unbind(int fd);
virtual void unbind(char* name, char* newname = "");

virtual void trace(int i = 1);
virtual void notrace();
virtual int  tracelevel() const;

virtual void quit();

// Multiple interpreters

virtual int safe() const;
virtual kit* slave(char* options = "");


// Anonymous dvbindings

virtual dvbinding* bind(handler* h); // anonymous

// Miscelaneous:

virtual void* interp() const; /// the interpreter, whatever it is
virtual void* tkwin() const;  /// returns the root window
virtual void* display() const; /// return the display
// AE with path to replace thewindow !

// System-level:

public: /// undocumented

virtual int mode() const;
virtual int mode(int fl);
virtual int mode(const char* = 0);

virtual int map(widget*,int i = 1) { return i; } /// to map or unmap widgets

// You can play numerous tricks with events - <b>obsolete</b>

virtual event* get() const;
virtual void put(event* p, int fl = 0); /// to update events

// Experimental -- role switching

kit* become(char* s, kit* k = 0);
void restore(kit* k = 0);

// Experimental

virtual void font(char*);
virtual int alias( char*, handler* );


private:
kit* tk;

public:
static kit* _default;
static void* _create;

static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static kit* lookup(const char*);
static void destroy();

};

extern HUSH_API kit* thekit();

#endif
