#ifndef TERM_H
#define TERM_H

#include <hush/export.h>
#include <iostream.h>


class HUSH_API term {
public:

enum { _nil, _constant, _variable, _compound, _higher };

term();
term(const char* f);
term(int v); /// variable
term(void* m); /// wildcard + assignment 
term(const char* f, const term* t1, const term* t2 = 0);
term(const term* t, const term* t1, const term* t2 = 0);
virtual ~term();

term(const term*);
term(const term&);

//term& operator=(const term& t);
//term& operator=(const term* t);

virtual char* operator()(); // a term as an iterator
virtual char* next();

// Output plolymorphic operators

operator int() { return ival(); }
operator char*() { return aval(); }

// Value types

virtual int ival() const;
virtual char* aval() const;

//// General

virtual char* category() const;
virtual int level() const;

virtual int type() const;
virtual term* arg(int i = 0) const;

virtual int iswild(const char* s = 0) const;

virtual int isnil(const char* s = 0) const;
virtual int isnum(const char* s = 0) const;
virtual int iscon(const char* s = 0) const;
virtual int isvar(const char* s = 0) const;
virtual int isfun(const char* s = 0) const;

virtual int has(const char* s = 0) const;
virtual int has(const term* t) const;

virtual int matches(const char* s) const;
virtual int matches(const term* t) const;


virtual term* varlist() const ;
//virtual char* varname();

virtual term* rename(int);
//virtual void substitute(int,term*);

virtual char* id() const ;
virtual char* str(int i = 0) const ;


//// Query
int isop(const char c) const;
int opin(const char* s) const;


//// Manip

//// Access

virtual char* fun() const;
virtual term* a1() const;
virtual term* a2() const;

void assign(void* v) { _val = v; }
void* value() { return _val; }

//// Lower

virtual term* body() const;

int exists() { return body()?1:0; }
term* self();

void inc();
void dec();
int refs() { return _ref; }

private:
void* _val;
int _ref;
term* _body;
};

inline ostream& operator<<(ostream& os, term& t) {
	os << (char*) t.str() << endl;
	return os;
	}

inline term* mkterm(const char* s) { return new term(s); }
inline term* mkterm(const char* f, const term* t1, const term* t2 = 0) { 
	return new term(f,t1,t2); 
	}

inline term* mkcon(const char* s) { return new term(s); }
inline term* mkvar(const char* s) { return new term(s); }
inline term* mkfun(const char* f, const term* t1, const term* t2 = 0) { 
	return new term(f,t1,t2); 
	}

inline term* mkc(const term* t1, const term* t2) { 
	return mkfun(",",t1,t2);
	}

inline term* mknil() { return mkcon("[]"); }
inline term* mklist(const term* t1, const term* t2) { 
	if (t1 && t2)
	        return mkfun(".",t1,t2);
	else if (t1) return (term*) t1;
	else if (t2) return (term*) t2;
	else return 0;
	}


//inline int type(const term& t) { return t.type(); }
inline int type(const term* t) { return t->type(); }

inline char* funof(const term& t) { return t.fun(); }
inline char* funof(const term* t) { return t->fun(); }

inline term* arg1of(const term& t) { return t.arg(1); }
inline term* arg1of(const term* t) { return t->arg(1); }

inline term* arg2of(const term& t) { return t.arg(2); }
inline term* arg2of(const term* t) { return t->arg(2); }

inline int iscon(const term& t) { return t.iscon(); }
inline int isnum(const term& t) { return t.isnum(); }
inline int isvar(const term& t) { return t.isvar(); }
inline int isfun(const term& t) { return t.isfun(); }

inline int iscon(const term* t) { return t->iscon(); }
inline int isnum(const term* t) { return t->isnum(); }
inline int isvar(const term* t) { return t->isvar(); }
inline int isfun(const term* t) { return t->isfun(); }


inline int isnull(const term&) { return 0; } 
inline int isnull(const term* t) { return t != 0; } 

inline int isc(const term& t) { return t.isfun(","); } 
inline int isc(const term* t) { return t->isfun(","); } 

inline int isnil(const term& t) { return t.iscon("[]"); } 
inline int isnil(const term* t) { return t->iscon("[]"); } 

inline int islist(const term& t) { return t.isfun("."); } 
inline int islist(const term* t) { return t->isfun("."); } 

inline int iszero(const term& t) { return t.iscon("0"); } 
inline int iszero(const term* t) { return t->iscon("0"); } 
#endif
