///Defines the abstract class <i>widget</i>.
///Abstract in a general sense, not in a technical C++ sense.
/// man:widget(4), <hush/kit.h>, <widgets/widgets.h>

#ifndef WIDGET_H
#define WIDGET_H

#include <hush/handler.h>
#include <hush/kit.h>

#include <hush/export.h>

class HUSH_API widget : public handler {
  public:
    widget(const char* s, const char* opts = 0, widget* ctx = 0);
    widget(const widget* w, const char* s, const char* ="", widget* =0);

    virtual widget* _create(const char* s, const char* opts = 0,
						widget* ctx = 0);
    virtual widget* _create(const widget* w, const char* s,
				    const char* ="", widget* =0);
    
    virtual ~widget();
    
    virtual char* info() const;
    virtual char* type() const ;
    
    virtual char* path() const ;

    virtual char* title() const;	/// get/set title (toplevel)
    virtual void title(const char*);

    virtual char* name() const ;
    
    virtual int eval(const char* s);
    virtual char* result() const;
    virtual char* evaluate(const char* cmd);
    
    virtual void destroy();

    virtual void cursor(const char*);
    virtual void image(const char*);
   
    virtual char* cget(const char*) const;
 
    virtual void configure(const char* s);
    virtual void geometry(int x, int y);
    
    virtual widget* pack(const char* options = "-fill x" );
    virtual widget* pack(widget* w, const char* options );
    
    virtual int bind(const char *s, handler* h, const char* args = "" );
    virtual int bind(handler* h, const char* args = "" );

    virtual int bind(const char *s, dvbinding* a,  const char* args = "" );
    virtual int bind(dvbinding* a,  const char* args = "" );

    virtual void source(const char* type, const char* args, const char* options="");
    virtual void target(const char* type, dvbinding* ac, const char* args, const char* options="");
    
    virtual void xscroll(const widget* s);
    virtual void yscroll(const widget* s);
    
    virtual void focus(const char* options = "");
    virtual void grab(const char* options = "");
    virtual void wait();
    
    virtual void* tkwin() const;
    
    virtual int width() const ;
    virtual int height() const;

//// To install bindings

protected: virtual void install(dvbinding* , const char*);
public:    virtual direct(const char*, dvbinding*, const char* = "" );

//// And to create meta widgets
    
protected:    void redirect(widget* w);
public:       virtual void alias( handler* caller = 0 ); // defaults to this

//// Invariant

public: int invariant() const;

//// To access the inner components of a compound widget
    
public:       
	widget* self() const;  // redirection
	int exists() const;    // but does it exist?

//// To support 'layered' widget components

private:      widget* _bdy; 
private:      widget* _this() const;  // _context() or _body() or this
private:      int _exists() const;

protected: 
        widget* _body() const;  // override for derived types   
	widget* _context() const; // override for derived types   
        widget* _self() const;    // (obscure) override for derived types   


    
protected:

	char* buf;
	
	widget();
	static int widgetno;
	
	virtual void init(const char* options);
	    
	void setpath( const char* s);
	void setpath( const widget* w, const char* s);

    
public:     int operator()();

    
private:
    char* _path;

protected:    char* thepath() const;  /// obsolete

public:
    
    // obsolete

    void font(char*);
    
    void prefix(const widget* p);

    virtual void place(const char* options); // obsolete
    
    virtual void map();
    virtual void unmap();

public:  // static things

static widget* _default;

static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static widget* lookup(const char*);
static void _destroy(); // clash otherwise
};


//<ignore>
#endif
//</ignore>
//<hr> api/hush/widget.c
