#ifndef TK_CANVAS_H
#define TK_CANVAS_H


#include <widgets/tk/export.h>

#include <widgets/hush/canvas.h>
#include <widgets/tk/tk.h>

class tk_canvas : public canvas {
public:

  char* category() const;
  
  tk_canvas();
  tk_canvas(const char *p, const char* options = "", canvas* = 0);
  tk_canvas(const widget* w, const char *p,
			const char* options = "", canvas* = 0);

  widget* _create(const char* s, const char* opts = 0,
						widget* ctx = 0);
  widget* _create(const widget* w, const char* s,
				    const char* ="", widget* =0);

  virtual ~tk_canvas();
  
  void region(int x1, int y1, int x2, int y2 );
  
  //enum { window_coords, canvas_coords }; // window_coords is default

  void coordsystem(int);
  int coordsystem() const;

  int canvasx(int screenx) const;
  int canvasy(int screeny) const;
  
  void install(dvbinding* ac, const char* args = "");
  
  void tag(int id, const char* tag);
  char* tags(int id) const;

  char* coords(int id) const;
  char* coords(const char* id) const;
 
  char* bbox(int id) const;
  char* bbox(const char* id) const;

  void del(int id);
  void del(const char* id);
  
  void move(int id, int x, int y);
  void move(const char* id, int x, int y);
  
  item* bitmap(int x1, int y1, const char* bitmap, const char* options="");
  void image(const char*) { } // to silence the compiler
  item* image(int x1, int y1, const char* bitmap, const char* options="");
  item* photo(int x1, int y1, int x2, int y2, const char* options ="");
  void readppm(int id, const char* file ="");
  
  item* line(int x1, int y1, int x2, int y2, const char* options ="");
  item* line(const char* linespec, const char* options ="");
  item* arc(int x1, int y1, int x2, int y2, const char* options ="");
  item* oval(int x1, int y1, int x2, int y2, const char* options ="");
  item* circle(int x1, int y1, int r, const char* options ="");
  item* polygon(const char* linespec, const char* options ="");
  item* rectangle(int x1, int y1, int x2, int y2, const char* options ="");
  item* text(int x1, int y1, const char* txt, const char* options="");
  item* window(int x1, int y1, const char* win, const char* options="");
  item* window(int x1, int y1, widget* win, const char* options="");
  
  void raise(const char* tag, const char* abovethis="");
  void lower(const char* tag, const char* belowthis="");
  
  /// obsolete or experimental
  
  item* sphere(int x1, int y1, int x2, int y2, const char* color="white", const char* tag = "sphere", const char* options ="");
  item* sphere(int x1, int y1, int r, const char* color="white", const char* tag = "sphere", const char* options ="");
  
  /// item* methods
  
  item* current();
  item* overlapping(int x, int y, const char* options);
  item* overlapping(int x, int y);
  
  char* items(int x, int y);
  char* items(int x1, int y1, int x2, int y2);
  
  void itemconfigure(int it, const char* options);
  void itemconfigure(const char* tag, const char* options);
  
  virtual itembind(int i, const char* s, dvbinding* a, const char* args = "" );
  virtual itembind(const char *i, const char* s, dvbinding* a, const char* args = "" );
  
  void fontmap(const char *xfont, const char *printerfont, int ptsize);
  void postscript(const char* file, const char* options="");
  
protected:
  virtual itemdirect(int i, const char *s, dvbinding* a, const char* args="" );
  virtual itemdirect(const char* i, const char *s, dvbinding* a, const char* args="" );

  virtual int convertx(int) const;
  virtual int converty(int) const;

private:
  int _cs; // coordinate system 
public:
static canvas* _default;
static int initialized;
static int initialize();
};

#endif
